
/* Tables used by TradingService Attribute Database  */

CREATE CACHED TABLE OF_Attributes
(
   Name CHAR (36) NOT NULL, /* name of the attribute */
   PID INTEGER NOT NULL, /* process id of this trader */
   Object INTEGER NOT NULL, /* value of the attribute */
   PRIMARY KEY (Name, PID)
)
;

/* Tables used by TradingService Service Database */

CREATE CACHED TABLE OF_Services
(
   Incarnation INTEGER NOT NULL, /* consecutive integer id number */
   PID INTEGER NOT NULL, /* process id of this trader */
   ServiceTypeIndex INTEGER NOT NULL, /* service type name index */
   Masked INTEGER NOT NULL, /* boolean value, whether service type is masked */
   InterfaceTypeIndex INTEGER NOT NULL, /* service interface name index */
   PRIMARY KEY (Incarnation, PID)
)
;

CREATE CACHED TABLE OF_ServiceProperties
(
   Incarnation INTEGER NOT NULL, /* consecutive integer id number */
   PID INTEGER NOT NULL, /* process id of this trader */
   NameIndex INTEGER NOT NULL, /* property name index */
   PMode INTEGER NOT NULL, /* propertymode as an integer */
   TypeCode BINARY NOT NULL /* serialized typecode */
)
;

CREATE CACHED TABLE OF_SuperTypes
(
   Incarnation INTEGER NOT NULL, /* consecutive integer id number */
   PID INTEGER NOT NULL, /* process id of this trader */
   ServiceType INTEGER NOT NULL /* incarnation number of a supertype */
)
;

/* Tables used by TradingService Offer Database */

CREATE CACHED TABLE OF_Offers
(
   OfferID CHAR (36) NOT NULL, /* offer id (UUID) */
   PID INTEGER NOT NULL, /* process id of this trader */
   IsProxy INTEGER NOT NULL, /* boolean, true if offer is a proxy, false otherwise */
   ServiceTypeIndex INTEGER NOT NULL, /* service type name index */
   OfferData BINARY NOT NULL, /* index of (object reference as a string) */
   MatchAll INTEGER NOT NULL, /*boolean, true if IF_MATCH_ALL is true */
   PRIMARY KEY (OfferID)
)
;

CREATE CACHED TABLE OF_OfferProps
(
   Name CHAR(50),  /* property name */
   Value CHAR(200),        /* property value */
   ServiceTypeIndex INTEGER NOT NULL,  /* service type name index */
   PID INTEGER NOT NULL,  /* process id of this trader */
   NumVal DOUBLE, /* numeric property value */
   OfferID CHAR(36) NOT NULL,  /* offer id ( UUID ) */
   ComplexValue INTEGER NOT NULL /* boolean, true if property value is dynamically evaluated, or is not a primative */
)
;

CREATE INDEX of_offerPropsIndexValue ON OF_OfferProps(Value, Name, ServiceTypeIndex, PID)
;

CREATE INDEX of_offerPropsIndexNumValue ON OF_OfferProps(NumVal, Name, ServiceTypeIndex, PID)
;

CREATE INDEX of_offerPropsIndexComplex ON OF_OfferProps(Name, ComplexValue)
;

CREATE INDEX of_offersIndex ON OF_Offers (ServiceTypeIndex, PID, MatchAll)
;
