/* Tables used by TradingService Attribute Database */

CREATE TABLE OF_Attributes
(
   Name CHAR(36) NOT NULL,  /* name of the attribute */
   PID INT8 NOT NULL,  /* process id of this trader */
   Object INT8 NOT NULL,  /* value of the attribute */
   PRIMARY KEY (Name, PID)
);

/* Tables used by TradingService Service Database */

CREATE TABLE OF_Services
(
   Incarnation INT8 NOT NULL,  /* consecutive integer id number */
   PID INT8 NOT NULL,  /* process id of this trader */
   ServiceTypeIndex INT8 NOT NULL,  /* index of service type name */
   Masked SMALLINT NOT NULL,  /* boolean value , whether service type is masked */
   InterfaceTypeIndex INT8 NOT NULL,  /* service interface name index */
   PRIMARY KEY (Incarnation, PID)
);

CREATE TABLE OF_ServiceProperties
(
   Incarnation INT8 NOT NULL,  /* consecutive integer id number */
   PID INT8 NOT NULL,  /* process id of this trader */
   NameIndex INT8 NOT NULL,  /* property name index */
   PMode INT8 NOT NULL,  /* propertymode as an integer */
   TypeCode BYTE NOT NULL /* serialized typecode */
);

CREATE TABLE OF_SuperTypes
(
   Incarnation INT8 NOT NULL,  /* consecutive integer id number */
   PID INT8 NOT NULL,  /* process id of this trader */
   ServiceType INT8 NOT NULL /* incarnation number of a supertype */
);

/* Tables used by TradingService Offer Database */

CREATE TABLE OF_Offers
(
   OfferID CHAR(36) NOT NULL,  /* offer id ( UUID ) */
   PID INT8 NOT NULL,  /* process id of this trader */
   IsProxy INT8 NOT NULL,  /* boolean , true if offer is a proxy , false otherwise */
   ServiceTypeIndex INT8 NOT NULL,  /* service type name index */
   OfferData BYTE NOT NULL,  /* object reference as a string */
   MatchAll INT8 NOT NULL, /*boolean, true if IF_MATCH_ALL is true */
   PRIMARY KEY (OfferID)
);

CREATE TABLE OF_OfferProps
(
   Name CHAR(50),  /* property name */
   Value CHAR(200),        /* property value */
   ServiceTypeIndex INT8 NOT NULL,  /* service type name index */
   PID INT8 NOT NULL,  /* process id of this trader */
   numVal NUMERIC,     /* property value when property type is number */
   OfferID CHAR(36) NOT NULL,  /* offer id ( UUID ) */
   ComplexValue INT8 NOT NULL /* boolean, true if property value is dynamically evaluated, or is not a primative */
);

CREATE INDEX of_offerPropsIndexValue ON OF_OfferProps(Value, Name, ServiceTypeIndex, PID);

CREATE INDEX of_offerPropsIndexNumValue ON OF_OfferProps(NumVal, Name, ServiceTypeIndex, PID);

CREATE INDEX of_offerPropsIndexComplex ON OF_OfferProps(Name, ComplexValue);

CREATE INDEX of_offersIndex ON OF_Offers (ServiceTypeIndex, PID, MatchAll);
