/* JNDI service tables */

CREATE TABLE OF_JNDI
(
   SID INTEGER NOT NULL,         /* server id uuid string */
   OID CHAR(36) NOT NULL,        /* object id uuid string */
   Name VARCHAR(255) NOT NULL,   /* name string */
   Class VARCHAR(255) NOT NULL,  /* classname */
   ChildUUID CHAR(36) NOT NULL,  /* leaf or context child uuid */
   PRIMARY KEY (SID, OID, Name)
)
go

CREATE TABLE OF_JNDI_Objects
(
   ChildUUID CHAR(36) NOT NULL,  /* object id uuid string */
   Object IMAGE NOT NULL,        /* serialized Object */
   PRIMARY KEY (ChildUUID)
)
go

CREATE PROCEDURE of_jndi_delete_context
   @ids INTEGER,
   @ido CHAR(36),
   @idc VARCHAR(255),
   @tag INTEGER output
AS
BEGIN
   UPDATE OF_JNDI SET ChildUUID = '00000000-0000-0000-0000-000000000000' WHERE ChildUUID = (SELECT ChildUUID FROM OF_JNDI WHERE SID = @ids AND OID = @ido AND Name = @idc)
   DELETE FROM OF_JNDI WHERE SID = @ids AND OID = @ido AND Name = @idc
   /* Retrieve number of rows processed by previous statement */
   SELECT @tag = @@rowcount
   RETURN 0
END
go

CREATE PROCEDURE of_jndi_delete_objects
   @ids INTEGER,
   @ido CHAR(36),
   @idc VARCHAR(255),
   @tag INTEGER output
AS
BEGIN
   DELETE FROM OF_JNDI_Objects WHERE ChildUUID = (SELECT ChildUUID FROM OF_JNDI WHERE SID = @ids AND OID = @ido AND Name = @idc)
   DELETE FROM OF_JNDI WHERE SID = @ids AND OID = @ido AND Name = @idc
   /* Retrieve number of rows processed by previous statement */
   SELECT @tag = @@rowcount
   RETURN 0
END
go

CREATE PROCEDURE of_jndi_insert_context
(
   @ids INTEGER,
   @ido CHAR(36),
   @idc VARCHAR(255),
   @classname VARCHAR(255),
   @child CHAR(36)
)
AS
BEGIN
   INSERT INTO OF_JNDI VALUES (@ids, @ido, @idc, @classname, @child)
   RETURN 0
END
go

CREATE PROCEDURE of_jndi_update_context
(
   @ids INTEGER,
   @ido CHAR(36),
   @idc VARCHAR(255),
   @classname VARCHAR(255),
   @child CHAR(36),
   @tag INTEGER
)
AS
BEGIN
   UPDATE OF_JNDI SET ChildUUID = @child, Class = @classname WHERE SID = @ids AND OID = @ido AND Name = @idc;
   /* Retrieve number of rows processed by previous statement */
   SELECT @tag = @@rowcount
   RETURN 0
END
go
