-- JTO/blobstore: table holding the sequence for the unique identifiers.
CREATE TABLE OF_Jto_ID
(
   JtoID NUMERIC (12,0) IDENTITY,
   field2 INTEGER
)
go

-- JTO/blobstore: table holding the JTO identifer/data.
CREATE TABLE OF_Jto
(
   JtoID VARCHAR(36) NOT NULL,
   Data IMAGE NOT NULL,
   PRIMARY KEY (JtoID)
)
go

-- Event database: table holding the stored events.
CREATE TABLE OF_Events
(
   EventID NUMERIC NOT NULL,
   ChannelID VARCHAR(36) NOT NULL,
   TStamp DATETIME NOT NULL,
   Data IMAGE NOT NULL,
   PRIMARY KEY (EventID, ChannelID)
)
go

-- Event database: table holding the pushing/pushed records.
CREATE TABLE OF_Push
(
   ProxyID VARCHAR(36) NOT NULL,
   EventID NUMERIC NOT NULL,
   ChannelID VARCHAR(36) NOT NULL,
   Status NUMERIC NOT NULL,
   PRIMARY KEY (ProxyID, ChannelID, EventID)
)
go

-- Event database: cascades deletes from OF_Jto to OF_Events.
CREATE TRIGGER of_delete_jto_events
ON OF_Jto
FOR DELETE AS
BEGIN
   DELETE OF_Events FROM OF_Events, deleted
      WHERE deleted.JtoID = OF_Events.ChannelID
END
go

-- Event database: cascades deletes from OF_Events to OF_Push.
CREATE TRIGGER of_delete_events_push
ON OF_Events
FOR DELETE AS
BEGIN
   DELETE OF_Push FROM OF_Push, deleted
      WHERE deleted.EventID = OF_Push.EventID AND deleted.ChannelID = OF_Push.ChannelID
END
go

-- JTO/blobstore: procedure to create a JTO record.
-- Not available: Sybase does not support TEXT/IMAGE parameters.

-- JTO/blobstore: procedure to update a JTO record.
-- Not available: Sybase does not support TEXT/IMAGE parameters.

-- JTO/blobstore: procedure to delete a JTO record.
CREATE PROCEDURE of_delete_jto
   @jid VARCHAR(36)
AS
BEGIN
   DELETE FROM OF_Jto WHERE JtoID = @jid
END
go

-- JTO/blobstore: procedure to delete all JTO records.
CREATE PROCEDURE of_delete_all_jtos
AS
BEGIN
   DELETE FROM OF_Jto
END
go

-- Event database: procedure to create an event record.
-- Not available: Sybase does not support TEXT/IMAGE parameters.

-- Event database: procedure to delete an event record.
CREATE PROCEDURE of_delete_event
   @eid NUMERIC,
   @cid VARCHAR(36)
AS
BEGIN
   DELETE FROM OF_Events WHERE EventID = @eid AND ChannelID = @cid
END
go

-- Event database: procedure to delete all event records for a channel.
CREATE PROCEDURE of_delete_events
   @cid VARCHAR(36)
AS
BEGIN
   DELETE FROM OF_Events WHERE ChannelID = @cid
END
go

-- Event database: procedure to delete all event records.
CREATE PROCEDURE of_delete_all_events
AS
BEGIN
   DELETE FROM OF_Events
END
go

-- Event database: procedure to create a push record.
CREATE PROCEDURE of_create_push
   @pid VARCHAR(36),
   @eid NUMERIC,
   @cid VARCHAR(36),
   @s NUMERIC
AS
BEGIN
   INSERT INTO OF_Push (ProxyID, EventID, ChannelID, Status) VALUES (@pid, @eid, @cid, @s)
END
go

-- Event database: procedure to update a push record.
CREATE PROCEDURE of_update_push
   @pid VARCHAR(36),
   @eid NUMERIC,
   @cid VARCHAR(36),
   @s NUMERIC
AS
BEGIN
   UPDATE OF_Push SET Status = @s WHERE ProxyID = @pid AND EventID = @eid AND ChannelID = @cid
END
go

-- Event database: procedure to delete a push record.
CREATE PROCEDURE of_delete_push
   @pid VARCHAR(36),
   @eid NUMERIC,
   @cid VARCHAR(36)
AS
BEGIN
   DELETE FROM OF_Push WHERE ProxyID = @pid AND EventID = @eid AND ChannelID = @cid
END
go

-- Event database: procedure to delete all push records.
CREATE PROCEDURE of_delete_all_push
AS
BEGIN
   DELETE FROM OF_Push
END
go

-- Set procedures to chained mode
sp_procxmode of_delete_jto, chained
go
sp_procxmode of_delete_all_jtos, chained
go
sp_procxmode of_delete_event, chained
go
sp_procxmode of_delete_events, chained
go
sp_procxmode of_delete_all_events, chained
go
sp_procxmode of_create_push, chained
go
sp_procxmode of_update_push, chained
go
sp_procxmode of_delete_push, chained
go
sp_procxmode of_delete_all_push, chained
go
