/*************************************************************************
 *                           - COPYRIGHT NOTICE -                        *
 *                                                                       *
 * The information contained herein and which follows is proprietary     *
 * information and is the property of PrismTech Corporation.  This       *
 * information is not to be divulged, released, copied, reproduced, or   *
 * conveyed to unauthorized personnel, companies, or other institutions  *
 * without the direct and expressed approval in writing of PrismTech     *
 * Corporation.                                                          *
 *                                                                       *
 *                                                                       *
 *                           - DISCLAIMER -                              *
 * Neither PrismTech Limited nor PrismTech Corporation ("PrismTech")     *
 * are under any obligation to support the use of this code which is     *
 * provided without warranty of any kind. PrismTech shall (except as     *
 * otherwise prohibited by law) have no liability whatsoever in relation *
 * to its use.                                                           *
 *                                                                       *
 ************************************************************************/

/**
 * @author PrismTech
 * @version Version 1.0
 */

package com.prismt.cos.CosClockService.examples;

import com.prismt.orb.*;
import org.omg.CosClockService.*;
import org.omg.CosClockService.ClockCatalogPackage.*;

/**
 * Resolve a reference to a ClockCatalog and display
 * the names of all entries.
 */
public class ClockCatalogQuery
{
    private org.omg.CORBA.ORB orb = null;
    private ClockCatalog catalog = null;
    private Clock local = null;
    private ControlledClock controlled = null;

    /**
     * Public constructor
     */
    public ClockCatalogQuery (org.omg.CORBA.ORB orb)
    {
        this.orb = orb;
    }

    /**
     * Query the ClockCatalog and display the names of any entries
     */
    public void query ()
    {
       try
       {
          // resolve a reference to the clock catalog
          org.omg.CORBA.Object ref = orb.resolve_initial_references ("ClockService");
          catalog = ClockCatalogHelper.narrow (ref);

          // obtain all catalog entries
          ClockEntry[] entries = catalog.available_entries ();

          // iterate through the catalog entries and display
          // the results
          System.out.println ("ClockCatalog Entries:\n");

          for (int i = 0; i < entries.length; i++)
          {
             System.out.println (" - " + entries [i].name);
          }
       }
       catch (org.omg.CORBA.ORBPackage.InvalidName invalidName)
       {
           System.out.println ("Failed to resolve clock catalog");
           invalidName.printStackTrace ();
       }
    }

    /**
     * Program entry point
     */
    public static void main (String[] args)
    {
       org.omg.CORBA.ORB orb = ObjectAdapter.init (args);
       ClockCatalogQuery query = new ClockCatalogQuery (orb);
       query.query ();
    }
}
