/*************************************************************************
 *                           - COPYRIGHT NOTICE -                        *
 *                                                                       *
 * The information contained herein and which follows is proprietary     *
 * information and is the property of PrismTech Corporation.  This       *
 * information is not to be divulged, released, copied, reproduced, or   *
 * conveyed to unauthorized personnel, companies, or other institutions  *
 * without the direct and expressed approval in writing of PrismTech     *
 * Corporation.                                                          *
 *                                                                       *
 *                                                                       *
 *                           - DISCLAIMER -                              *
 * Neither PrismTech Limited nor PrismTech Corporation ("PrismTech")     *
 * are under any obligation to support the use of this code which is     *
 * provided without warranty of any kind. PrismTech shall (except as     *
 * otherwise prohibited by law) have no liability whatsoever in relation *
 * to its use.                                                           *
 *                                                                       *
 ************************************************************************/

/**
 * @author PrismTech
 * @version Version 1.0
 */

package com.prismt.cos.CosClockService.examples;

import com.prismt.orb.*;
import org.omg.CosClockService.*;
import org.omg.CosPropertyService.*;

/**
 * Query a controlled clock
 */
public class GenericControlledClockQuery
{
    private org.omg.CORBA.ORB orb = null;
    private ControlledClock controlledClock = null;

    /**
     * Public constructor
     */
    public GenericControlledClockQuery (org.omg.CORBA.ORB orb)
    {
        this.orb = orb;
    }

    /**
     * Get the current time for a Controlled Clock
     */
    public void query ()
    {
       try
       {
           org.omg.CORBA.Object ref = orb.resolve_initial_references ("ControlledClock");
           controlledClock = ControlledClockHelper.narrow (ref);

           // display the time
           System.out.println ("Current time: " + controlledClock.current_time ());
       }
       catch (org.omg.CORBA.ORBPackage.InvalidName invalidName)
       {
           System.out.println ("Failed to resolve ControlledClock");
           invalidName.printStackTrace ();
       }
       catch (TimeUnavailable unavailable)
       {
           System.out.println ("Time unavailable: ");
           unavailable.printStackTrace ();
       }
    }

    /**
     * Program entry point
     */
    public static void main (String[] args)
    {
       org.omg.CORBA.ORB orb = ObjectAdapter.init (args);
       GenericControlledClockQuery query = new GenericControlledClockQuery (orb);

       query.query ();
    }
}
