/*************************************************************************
 *                           - COPYRIGHT NOTICE -                        *
 *                                                                       *
 * The information contained herein and which follows is proprietary     *
 * information and is the property of PrismTech Corporation.  This       *
 * information is not to be divulged, released, copied, reproduced, or   *
 * conveyed to unauthorized personnel, companies, or other institutions  *
 * without the direct and expressed approval in writing of PrismTech     *
 * Corporation.                                                          *
 *                                                                       *
 *                                                                       *
 *                           - DISCLAIMER -                              *
 * Neither PrismTech Limited nor PrismTech Corporation ("PrismTech")     *
 * are under any obligation to support the use of this code which is     *
 * provided without warranty of any kind. PrismTech shall (except as     *
 * otherwise prohibited by law) have no liability whatsoever in relation *
 * to its use.                                                           *
 *                                                                       *
 ************************************************************************/

/**
 * @author PrismTech
 * @version Version 1.0
 */

package com.prismt.cos.CosClockService.examples;

import com.prismt.orb.*;
import org.omg.CosClockService.*;

import com.prismt.cos.CosClockService.UTCValueFactoryImpl;

/**
 * Resolve a reference to the UtcTimeService
 */
public class UtcTimeServiceQuery
{
    private org.omg.CORBA.ORB orb = null;
    private UtcTimeService localClock = null;

    /**
     * Public constructor
     */
    public UtcTimeServiceQuery (org.omg.CORBA.ORB orb)
    {
       this.orb = orb;
    }

    /**
     * Query the UtcTimeService and display the universal time
     */
    public void query ()
    {
       try
       {
           // register the UTC ValueFactory implementation so that UTC values
           // can be unmarshalled by the ORB
           ((org.omg.CORBA_2_3.ORB) orb).register_value_factory (UTCHelper.id (), new UTCValueFactoryImpl ());

           // resolve a reference to the local clock
           org.omg.CORBA.Object ref = orb.resolve_initial_references ("LocalClock");
           localClock = UtcTimeServiceHelper.narrow (ref);

           // display the universal time
           UTC universal = localClock.universal_time ();
           System.out.println ("Universal time: \n");

           System.out.println (" Time : " + universal.time);
           System.out.println (" Inaccuracy (inacchi + inacclo)  : " + universal.inaccuracy ());
           TimeSpan tSpan = universal.interval ();
           System.out.println (" TimeSpan lower bound (time - inaccuracy) : " + tSpan.lower_bound);
           System.out.println (" TimeSpan upper bound (time + inaccuracy) : " + tSpan.upper_bound);

       }
       catch (org.omg.CORBA.ORBPackage.InvalidName invalidName)
       {
           System.out.println ("Failed to resolve UtcTimeService");
           invalidName.printStackTrace ();
       }
       catch (TimeUnavailable unavailable)
       {
           System.out.println ("Time unavailable: ");
           unavailable.printStackTrace ();
       }
    }

    /**
     * Program entry point
     */
    public static void main (String[] args)
    {
       org.omg.CORBA.ORB orb = ObjectAdapter.init (args);
       UtcTimeServiceQuery query = new UtcTimeServiceQuery (orb);

       query.query ();
    }
}
