package com.prismt.cos.CosNaming.examples;


import com.prismt.orb.ORBAdapter;
import com.prismt.orb.portable.stream.SerializableObject;
import java.util.Enumeration;
import java.util.Hashtable;
import javax.naming.Context;
import javax.naming.NameClassPair;
import javax.naming.NamingException;
import javax.naming.spi.NamingManager;
import org.omg.CosNaming.NamingContextExt;
import org.omg.CosNaming.NamingContextExtHelper;


public class JNDItoCORBA
{
   public static void main (String[] args)
   {
      // This program does require the UUID and SID. This is because we are
      // using the same data as that written by the naming service.
      // The easiest way to discover the UUID and SID that the service is using
      // as the root context is to enable Info logging for the Naming Service.
      // Then the following lines will appear in the log file:
      //
      // "
      // INFO - Process ID: 0ba57cb0-4dae-11d4-ada7-ce8c9fa68378
      // INFO - Server  ID: 0ba57cb0-4dae-11d4-ada7-ce8c9fa68378
      // INFO - Common: database = com.prismt.jdbc.Database@16304c8
      // INFO - initSID: database = com.prismt.jdbc.Database@16304c8
      // INFO - NamingService UUID is 0d68b080-4dae-11d4-ada7-ce8c9fa68378
      // "
      //
      // Here, we need the second and fifth line. These two values may be passed
      // in as args[0] and args[1] respectively.  These instructions are for
      // JDBC. If file based persistence is to be used the same principles
      // apply.

      if (args.length != 2)
      {
         System.out.println ("Usage JNDItoCORBA UUID SID");
         System.exit (1);
      }


      Context   initialctx = null;
      Context   subctx  = null;
      Hashtable env     = new Hashtable ();

      // Set inital context
      env.put (javax.naming.Context.INITIAL_CONTEXT_FACTORY,
              "com.prismt.j2ee.jndi.OpenFusionCtxFactory");

      // Use JDBC based persistence
      //
      // ALTER THESE VALUES
      env.put
         ("com.prismt.j2ee.jndi.OpenFusionSPI.JDBC.URL", "jdbc:hsqldb:/tmp");
      env.put
      (
         "com.prismt.j2ee.jndi.OpenFusionSPI.JDBC.User",
         "sa"
      );
      // Use a read cache. Accept the defaults for read limit.
      env.put ("com.prismt.j2ee.jndi.OpenFusionSPI.Cache", "ReadCache");

      // Set the UUID.
      env.put ("com.prismt.j2ee.jndi.OpenFusionSPI.UUID",
               args[0]);
      // Set the SID.
      env.put ("com.prismt.j2ee.jndi.OpenFusionSPI.SID",
               args[1]);




      // Setup some variables
      org.omg.CORBA.ORB orb = ORBAdapter.init (args);
      NamingContextExt rootContext = null;

      try
      {
         // Resolve NameService
         org.omg.CORBA.Object obj =
            orb.resolve_initial_references ("NameService");
         rootContext = NamingContextExtHelper.narrow (obj);
      }
      catch (org.omg.CORBA.ORBPackage.InvalidName ex)
      {
         System.err.println ("Unknown Service");
         System.exit (1);
      }


      try
      {
         // Get the root context
         initialctx = NamingManager.getInitialContext (env);
      }
      catch (NamingException e)
      {
         e.printStackTrace ();
         System.err.println ("Caught: " + e);
         System.exit (1);
      }

      System.out.println ("List the current root context contents");
      try
      {
         Enumeration e = initialctx.list ("");

         while (e.hasMoreElements ())
         {
            NameClassPair np = (NameClassPair)e.nextElement ();
            System.out.println
            (
               "\tFound name " + np.getName () +
               " of class " +  np.getClassName ()
            );
            System.out.println
               ("\tResolve: " + initialctx.lookup (np.getName ()));
         }
      }
      catch (NamingException e)
      {
         e.printStackTrace ();
         System.err.println ("Caught: " + e);
      }


      System.out.println ("Create a sub context NEWCTX");
      try
      {
         initialctx.createSubcontext ("NEWCTX");
      }
      catch (NamingException e)
      {
         e.printStackTrace ();
         System.err.println ("Caught: " + e);
      }


      System.out.println ("Bind Corba Object");
      try
      {
         initialctx.bind ("CorbaObject", new SerializableObject (rootContext));
      }
      catch (NamingException e)
      {
         e.printStackTrace ();
         System.err.println ("Caught: " + e);
      }


      System.out.println ("Bind new string object ");
      try
      {
         initialctx.bind ("StringObject", new String ("A string to store"));
      }
      catch (NamingException e)
      {
         e.printStackTrace ();
         System.err.println ("Caught: " + e);
      }


      System.out.println ("List the current root context contents");
      try
      {
         Enumeration e = initialctx.list ("");

         while (e.hasMoreElements ())
         {
            NameClassPair np = (NameClassPair)e.nextElement ();
            System.out.println
            (
               "\tFound name " + np.getName () +
               " of class " +  np.getClassName ()
            );
            System.out.println
               ("\tResolve: " + initialctx.lookup (np.getName ()));
         }
      }
      catch (NamingException e)
      {
         e.printStackTrace ();
         System.err.println ("Caught: " + e);
      }
   }
}
