/*************************************************************************
 *                           - COPYRIGHT NOTICE -                        *
 *                                                                       *
 * The information contained herein and which follows is proprietary     *
 * information and is the property of PrismTech Corporation.  This       *
 * information is not to be divulged, released, copied, reproduced, or   *
 * conveyed to unauthorized personnel, companies, or other institutions  *
 * without the direct and expressed approval in writing of PrismTech     *
 * Corporation.                                                          *
 *                                                                       *
 *                                                                       *
 *                           - DISCLAIMER -                              *
 * Neither PrismTech Limited nor PrismTech Corporation ("PrismTech")     *
 * are under any obligation to support the use of this code which is     *
 * provided without warranty of any kind. PrismTech shall (except as     *
 * otherwise prohibited by law) have no liability whatsoever in relation *
 * to its use.                                                           *
 *                                                                       *
 ************************************************************************/

/**
 * @author PrismTech
 * @version Version 1.0
 */

package com.prismt.cos.CosNaming.examples.LoadBalance;

import org.omg.CosNaming.*;
import org.omg.CosNaming.NamingContextPackage.*;

import com.prismt.orb.ORBAdapter;

/**
 * <code>LBClient</code> is an example of using the NameService and
 * LoadBalancing
 *
 */
public class LBClient
{
   /**
    * <code>main</code> will create a new LBClient that will resolve the
    * NameService and perform some operations on it.
    *
    * @param args[] a <code>String</code> value
    */
   public static void main (String args[])
   {
      System.out.println ("Initialising the orb.");
      org.omg.CORBA.ORB orb = ORBAdapter.init (args);

      LBClient lbc = new LBClient (orb);
   }


   /**
    * Creates a new <code>LBClient</code> instance.
    *
    * @param orb an <code>org.omg.CORBA.ORB</code> value
    */
   public LBClient (org.omg.CORBA.ORB orb_)
   {
      org.omg.CORBA.ORB    orb         = null;
      org.omg.CORBA.Object nameRef     = null;
      org.omg.CORBA.Object obj         = null;
      NamingContext        rootContext = null;
      NameComponent        lbAlias[]   = null;
      Server               server      = null;

      orb = orb_;

      try
      {
         System.out.println ("Resolving name service.");
         nameRef = orb.resolve_initial_references ("NameService");
      }
      catch (org.omg.CORBA.ORBPackage.InvalidName in)
      {
         System.out.println ("Could not resolve Name Service");
         System.exit (1);
      }

      System.out.println ("Narrowing Name Service.");
      rootContext = NamingContextHelper.narrow (nameRef);

      lbAlias = new NameComponent[1];
      lbAlias[0] = new NameComponent ();
      lbAlias[0].id = "LoadBalancingAlias";
      lbAlias[0].kind = "LoadBalancerExample";


      System.out.println ("Starting to resolve server objects");
      try
      {
         for (int i = 0; i < 5; i++)
         {
            System.out.println ("Resolving LoadBalancing Alias.");
            obj = rootContext.resolve (lbAlias);
            System.out.println ("Resolved LoadBalancingAlias" + obj);
            server = ServerHelper.narrow (obj);
            System.out.println ("Got Server Object named " + server.getName ());
         }
      }
      catch (CannotProceed e)
      {
         System.out.println ("Resolve Failed: " + e);
         e.printStackTrace ();
         System.exit (1);
      }
      catch (InvalidName e)
      {
         System.out.println ("Resolve Failed: " + e);
         e.printStackTrace ();
         System.exit (1);
      }
      catch (NotFound e)
      {
         System.out.println ("Resolve Failed: " + e);
         e.printStackTrace ();
         System.exit (1);
      }

      System.out.println ("Completed successfully");
   }
}
