package com.prismt.cos.CosNaming.examples;

import com.prismt.cos.CosNaming.NamingExtensions.*;
import com.prismt.cos.CosNaming.NamingExtensions.LoadBalancerPackage.*;
import org.omg.CORBA.*;
import org.omg.CosNaming.*;
import org.omg.CosNaming.NamingContextPackage.*;

public class LoadBalancing
{
   public static void main (String[] args)
   {
      // Setup some variables
      NamingContextExt     rootContext = null;
      org.omg.CORBA.ORB    orb         = null;
      org.omg.CORBA.Object obj         = null;
      NamingContext        ctx1        = null;
      NamingContext        ctx2        = null;
      NamingContext        ctx3        = null;
      NameComponent[]      lbobj       = null;
      NameComponent[]      lbalias     = null;
      LoadBalancer         lb          = null;
      LoadBalancingFactory lbfactory   = null;

      orb = com.prismt.orb.ObjectAdapter.init (args);
      com.prismt.orb.ObjectAdapter.ready (false);

      // Initialise the Name Service
      try
      {
         obj = orb.resolve_initial_references ("NameService");
         rootContext = NamingContextExtHelper.narrow (obj);
      }
      catch (Exception e)
      {
         System.err.println ("Unable to resolve NameService:" + e);
         System.exit (1);
      }


      // Initialise the Load Balancing Factory
      try
      {
         obj = orb.resolve_initial_references ("LoadBalancingFactory");
         lbfactory = LoadBalancingFactoryHelper.narrow (obj);
      }
      catch (Exception e)
      {
         System.out.println ("Unable to resolve LoadBalancingFactory: " + e);
         System.exit (1);
      }


      // Create some contexts that we can place in the Load Balancer
      System.out.println ("Creating contexts...");
      ctx1 = rootContext.new_context ();
      ctx2 = rootContext.new_context ();
      ctx3 = rootContext.new_context ();
      System.out.println ("ctx1: " + ctx1);
      System.out.println ("ctx2: " + ctx2);
      System.out.println ("ctx3: " + ctx3);


      // Create a Load Balancer specifying the RoundRobin policy
      try
      {
         lb = lbfactory.createLoadBalancer ("Roundrobin");
         lb.add (ctx1);
         lb.add (ctx2);
         lb.add (ctx3);
      }
      catch (InvalidPolicy e)
      {
         System.out.println ("Caught invalid policy: " + e);
         System.exit (1);
      }
      catch (InvalidPlugin e)
      {
         System.out.println ("Caught invalid plugin: " + e);
         System.exit (1);
      }


      // Create the name components to bind the load balancer into the service.
      lbalias = new NameComponent[1];
      lbalias[0] = new NameComponent ();
      lbalias[0].id = "LoadBalancingAlias";
      lbalias[0].kind = "Example1";
      lbobj = new NameComponent[1];
      lbobj[0] = new NameComponent ();
      lbobj[0].id = "LoadBalancingObject";
      lbobj[0].kind = "Example1";

      System.out.println ("Bind Load Balancer");
      try
      {
         // Note we are binding both alias and the management object here
         // lbobj and lbalias are NameComponents.
         rootContext.bind (lbobj, lb);
         rootContext.bind (lbalias, lb.getAlias ());
      }
      // Bind throws NotFound, CannotProceed, InvalidName or AlreadyBound.
      catch (Exception e)
      {
         System.out.println ("Caught exception in bind: " + e);
         System.exit (1);
      }
      // Set it to null as it will be accessed through the name service now.
      lb = null;


      // Resolve the objects
      //
      // The sequence will be 1,2,3,1.
      System.out.println ("Attempt to resolve the objects");
      try
      {
         for (int i=0; i<4; i++)
         {
            obj = rootContext.resolve (lbalias);

            System.out.println ("Resolved: " + obj);
            try
            {
               NamingContextExt ctx = NamingContextExtHelper.narrow (obj);
               System.out.println ("Resolved name context: " + ctx);
            }
            catch (org.omg.CORBA.BAD_PARAM e)
            {
               System.err.println
               (
                  "Unable to narrow the object. Maybe LoadBalancing " +
                  "is not enabled in the name service?"
               );
               break;
            }
         }
      }
      catch (Exception e)
      {
         System.out.println ("Caught exception: " + e);
         System.exit (1);
      }


      // Retrieve the LoadBalancer object. The NamingService will return
      // the actual LoadBalancer in this case. This allows one to modify the
      // LoadBalancer e.g. add/remove objects, reset the policy etc.
      System.out.println ("Retrieve the management object");
      try
      {
         org.omg.CORBA.Object newlb = rootContext.resolve (lbobj);
         lb = LoadBalancerHelper.narrow (newlb);

         // List
         org.omg.CORBA.Object initiallist[] = lb.list ();
         org.omg.CORBA.Object anobject = initiallist[initiallist.length / 2];

         // Objects may be retrieved (according to policy) via the get op.
         System.out.println ("LoadBalancer retrieved: " + lb.get ());

         System.out.println
            ("Initial size of Load Balancer: " + initiallist.length);

         // Remove
         System.out.println ("Remove an object");
         lb.remove (anobject);

         org.omg.CORBA.Object elements[] = lb.list ();
         System.out.println ("New size of Load Balancer: " + elements.length);
      }
      catch (Exception e)
      {
         System.out.println ("Caught exception: " + e);
         System.exit (1);
      }


      // Add a plugin (the example class below)
      System.out.println ("Adding the TestPlugin");
      try
      {
         lb.addPlugin
            ("com.prismt.cos.CosNaming.examples.LoadBalancingTestPlugin");
      }
      catch (InvalidPlugin e)
      {
         System.out.println ("Caught exception: " + e);
         System.exit (1);
      }


      // Set the policy to that of the new plugin.
      System.out.println ("Setting a new policy");
      try
      {
         lb.setPolicy ("TEST_POLICY");
      }
      catch (InvalidPolicy e)
      {
         System.out.println ("Caught exception: " + e);
         System.exit (1);
      }

      // Resolve
      //
      // This resolve should always print out the second object - i.e. both
      // resolves will be the same.
      System.out.println ("Attempt to resolve the objects");
      try
      {
         for (int i=0; i<2; i++)
         {
            obj = rootContext.resolve (lbalias);

            System.out.println ("Resolved: " + obj);
            try
            {
               NamingContextExt ctx = NamingContextExtHelper.narrow (obj);
               System.out.println ("Resolved rootCtx: " + ctx);
            }
            catch (org.omg.CORBA.BAD_PARAM e)
            {
               System.err.println ("Unable to narrow the object. Maybe LoadBalancing is not enabled in the name service?");
               break;
            }
         }
      }
      catch (Exception e)
      {
         System.out.println ("Caught exception: " + e);
         System.exit (1);
      }
      System.exit (0);
   }
}
