package com.prismt.cos.CosNaming.examples;

import org.omg.CosNaming.*;
import org.omg.CosNaming.NamingContextPackage.*;
import org.omg.CosNaming.NamingContextExt.*;
import org.omg.CosNaming.NamingContextExtPackage.*;
import com.prismt.orb.util.IORDecoder;
import com.prismt.orb.ORBAdapter;
import com.prismt.orb.*;
import com.prismt.util.StringUtil;

public class Tutorial
{
   public static void main (String args[])
   {
      org.omg.CORBA.Object obj = null;
      org.omg.CORBA.ORB orb = null;
      NamingContextExt rootContext = null;

      orb = ObjectAdapter.init (args);

      try
      {
         obj = orb.resolve_initial_references ("NameService");
         rootContext = NamingContextExtHelper.narrow (obj);
      }
      catch (org.omg.CORBA.ORBPackage.InvalidName ex)
      {
         System.err.println ("Failed to resolve NameService");
         System.exit (1);
      }

      try
      {
         NameComponent newName[] = new NameComponent[1];
         // set id field to "example" and kind field to an empty string
         newName[0] = new NameComponent ("example", "");

         org.omg.CORBA.Object demoObject = rootContext;

         // obtain demoObject reference ...

         rootContext.bind (newName, demoObject);
      }
      catch (org.omg.CORBA.UserException ex)
      {
         System.err.println ("caught " + ex);
      }

      try
      {
         org.omg.CORBA.Object demoObject;
         demoObject = rootContext;

         NameComponent newName[] = new NameComponent[1];

         // set id field to "example" and kind field to an empty string
         newName[0] = new NameComponent ("example name", "");

         rootContext.bind (newName, demoObject);
      }
      catch (org.omg.CORBA.UserException ex)
      {
         System.err.println ("caught " + ex);
      }

      try
      {
         BindingIteratorHolder bi = new BindingIteratorHolder ();
         BindingListHolder bl = new BindingListHolder ();
         NamingContextExt childContext = null;

         rootContext.list (0, bl, bi);

         BindingHolder binding = new BindingHolder ();
         while (bi.value != null && bi.value.next_one (binding))
         {
            try
            {
               obj = rootContext.resolve (binding.value.binding_name);
               if (binding.value.binding_type == BindingType.ncontext)
               {
                  childContext = NamingContextExtHelper.narrow (obj);
                  // do something with childContext
               }
               else
               {
                  // do something with obj
               }
            }
            catch (org.omg.CORBA.UserException ex)
            {
               System.err.println ("resolve exception " + ex);
            }
         }
      }
      catch (Exception ex)
      {
         System.err.println ("caught " + ex);
      }

      try
      {
         NamingContext namingContextObject = rootContext;

         NameComponent newName[] = new NameComponent[1];
         newName[0] = new NameComponent ("example2", "");

         rootContext.bind_context (newName, namingContextObject);
      }
      catch (org.omg.CORBA.UserException ex)
      {
         System.err.println ("caught " + ex);
      }

      try
      {
         NameComponent newName[] = new NameComponent[2];
         newName[0] = new NameComponent ("example2", "");
         newName[1] = new NameComponent ("example3", "context");

         NamingContext newContext = rootContext.bind_new_context (newName);
      }
      catch (org.omg.CORBA.UserException ex)
      {
         System.err.println ("caught " + ex);
      }

      try
      {
         NameComponent comp[] = new NameComponent[2];
         comp[0] = new NameComponent ("example2", "");
         comp[1] = new NameComponent ("example3", "context");

         rootContext.unbind (comp);
      }
      catch (org.omg.CORBA.UserException ex)
      {
         System.err.println ("caught " + ex);
      }

      try
      {
         NamingContext newContext = rootContext.new_context ();

         newContext.destroy ();
      }
      catch (org.omg.CORBA.UserException ex)
      {
         System.err.println ("caught " + ex);
      }

      try
      {
         NameComponent comp[] = new NameComponent[1];
         comp[0] = new NameComponent ("example", "");
         obj = rootContext.resolve (comp);
      }
      catch (org.omg.CORBA.UserException ex)
      {
         System.err.println ("caught " + ex);
      }

      try
      {
         BindingIteratorHolder iter = new BindingIteratorHolder ();
         BindingListHolder list = new BindingListHolder ();
         rootContext.list (5, list, iter);

         for (int i = 0; i < list.value.length; i++)
         {
            System.out.println ("list entry " + i);
            System.out.print   (" name length: ");
            System.out.println (list.value[i].binding_name.length);
            System.out.print   (" name id: ");
            System.out.println (list.value[i].binding_name[0].id);
            System.out.print   (" name kind: ");
            System.out.println (list.value[i].binding_name[0].kind);
            System.out.print   (" bind type: ");
            System.out.println (list.value[i].binding_type);
         }

         BindingHolder binding = new BindingHolder ();

         while (iter.value != null && iter.value.next_one (binding))
         {
            obj = rootContext.resolve (binding.value.binding_name);
         }
      }
      catch (org.omg.CORBA.UserException ex)
      {
         System.err.println ("caught " + ex);
      }

      // Interoperable Naming Examples
      NamingContextExt rootExtContext = null;

      try
      {
         obj = orb.resolve_initial_references ("NameService");
         rootExtContext = NamingContextExtHelper.narrow (obj);
      }
      catch (org.omg.CORBA.ORBPackage.InvalidName ex)
      {
         System.err.println ("Failed to resolve NameService");
         System.exit (1);
      }

      if (rootExtContext == null)
      {
         System.err.println ("Failed to resolve NameService");
         System.exit (1);
      }
      // We know that 'example' has been bound into the NameService together
      // with 'example2' subcontext. Therefore attempt to resolve using
      // stringified names.
      try
      {
         org.omg.CORBA.Object res;
         NameComponent newName[] = new NameComponent[2];
         newName[0] = new NameComponent ("example2", "");
         newName[1] = new NameComponent ("example2", "");

         String stringified = new String (rootExtContext.to_string (newName));
         System.out.println ("Stringified name is: " + stringified);

         try
         {
            res = rootExtContext.resolve_str (stringified);

            if (res != null)
            {
               System.out.println ("Object: " + res.toString ());
            }
         }
         catch (org.omg.CORBA.UserException ex)
         {
            System.out.println ("Resolve Exception: " + ex);
         }

         try
         {
            NameComponent copy[] = rootExtContext.to_name (stringified);
            org.omg.CORBA.Object copyobj = rootExtContext.resolve (copy);

            if (copyobj!=null)
            {
               System.out.println ("Object:" + copyobj.toString ());
            }
         }
         catch (org.omg.CORBA.UserException ex)
         {
            System.out.println ("Resolve Exception: " + ex);
         }

         try
         {
            // The resulting URL address can then be used to resolve within
            // a naming service.
            System.out.println
            (
               "to_url: "
               + rootExtContext.to_url ("rir:", stringified)
            );
         }
         catch (org.omg.CosNaming.NamingContextExtPackage.InvalidAddress e)
         {
            System.out.println ("Ex: " + e);
         }

      }
      catch (org.omg.CosNaming.NamingContextPackage.InvalidName ex)
      {
         System.out.println ("Name Exception: " + ex);
      }

      try
      {
         // The Corbaloc string that is generated can be used to resolve the
         // service.
         System.out.println ("Root IOR: " + orb.object_to_string (rootContext));

         // These operations are OpenFusion specific.

         NamingContextExt newCtx = null;
         IORDecoder decoder = new IORDecoder (rootContext);
         StringBuffer locstr = new StringBuffer ("corbaloc::");

         locstr.append (decoder.getHost ());
         locstr.append (":");
         locstr.append (decoder.getPort ());
         locstr.append ("/");
         locstr.append
            (StringUtil.encode (StringUtil.byteToString (decoder.getKey ())));

         // StringBuffer locstr now contains the address. Attempt to resolve
         // to check.
         // Cannot use orb.string_to_object as no hooks are available to
         // add support for INS extensions.
         newCtx = NamingContextExtHelper.narrow
            (ORBAdapter.stringToObject (locstr.toString ()));

         if (newCtx != null)
         {
            System.out.println
            (
               "Successfully resolved context: " +
               ORBAdapter.objectToString (newCtx)
            );
         }
         else
         {
            System.err.println ("Failed to resolve NameService");
         }
      }
      catch (Exception ex)
      {
         System.err.println ("Unable to resolve NameService" + ex);
         ex.printStackTrace ();
      }

      try
      {
         // The Corbaname string that is generated can be used to resolve the
         // service.
         System.out.println ("Root IOR: " + orb.object_to_string (rootContext));

         // These operations are OpenFusion specific.

         NamingContextExt newCtx = null;
         IORDecoder decoder = new IORDecoder (rootContext);
         StringBuffer corbaname = new StringBuffer ("corbaname:iiop:");

         corbaname.append (decoder.getHost ());
         corbaname.append (":");
         corbaname.append (decoder.getPort ());
         corbaname.append ("/");
         corbaname.append
            (StringUtil.encode (StringUtil.byteToString (decoder.getKey ())));
         corbaname.append ("#example name");

         // StringBuffer corbaname now contains the NamingContext. Attempt to
         // resolve to check.
         // Cannot use orb.string_to_object as no hooks are available to
         // add support for INS extensions.
         newCtx = NamingContextExtHelper.narrow
            (ORBAdapter.stringToObject (corbaname.toString ()));

         if (newCtx != null)
         {
            System.out.println
               ("Context IOR: " + ORBAdapter.objectToString (newCtx));
         }
         else
         {
            System.err.println ("Failed to resolve NameService");
         }
      }
      catch (Exception ex)
      {
         System.err.println ("Exception caught: " + ex);
         ex.printStackTrace ();
      }
   }
}
