/*************************************************************************
 *                           - COPYRIGHT NOTICE -                        *
 *                                                                       *
 * The information contained herein and which follows is proprietary     *
 * information and is the property of PrismTech Corporation.  This       *
 * information is not to be divulged, released, copied, reproduced, or   *
 * conveyed to unauthorized personnel, companies, or other institutions  *
 * without the direct and expressed approval in writing of PrismTech     *
 * Corporation.                                                          *
 *                                                                       *
 *                                                                       *
 *                           - DISCLAIMER -                              *
 * Neither PrismTech Limited nor PrismTech Corporation ("PrismTech")     *
 * are under any obligation to support the use of this code which is     *
 * provided without warranty of any kind. PrismTech shall (except as     *
 * otherwise prohibited by law) have no liability whatsoever in relation *
 * to its use.                                                           *
 *                                                                       *
 ************************************************************************/

/**
 * @author PrismTech
 * @version Version 1.0
 */

package com.prismt.cos.CosNotification.examples.Benchmark;

import java.io.*;

import org.omg.CosNotification.*;
import org.omg.CosNotifyComm.*;
import org.omg.CosNotifyChannelAdmin.*;
import org.omg.CosNotifyFilter.*;


/**
 * The benchmark Any/untyped push consumer.
 */
public class BenchmarkAnyConsumer extends BenchmarkConsumer
                                  implements org.omg.CosEventComm.PushConsumerOperations,
                                             java.io.Serializable
{
   private ProxyPushSupplier proxySupplier = null;
   private org.omg.CosEventComm.PushConsumer consumer = null;


   /**
    * Main method.  Creates an instance of the consumer and waits for events.
    * @param args  The command line arguments.
    */
   public static void main (String[] args)
   {
      BenchmarkAnyConsumer cons = new BenchmarkAnyConsumer ();
      cons.initialise (args);
      cons.connect ();

      System.out.println (cons + ": ready: press return to exit");
      BufferedReader br =
         new BufferedReader (new InputStreamReader (System.in));
      try
      {
         br.readLine ();
      }
      catch (IOException ioe)
      {
      }

      // If reporting, always display the final rate.
      if (cons.reportThreshold != 0)
      {
         if (cons.lastReportEventCount != cons.eventsReceived)
         {
            cons.report (cons.eventsReceived);
         }
      }
      
      cons.disconnect ();
      System.exit (0);
   }


   public BenchmarkAnyConsumer ()
   {
      super ();
      isSupplier = false;
      clientType = ClientType.ANY_EVENT;
      setID ("BenchmarkAnyConsumer");
   }


   public void setProxyReference (org.omg.CORBA.Object proxy)
   {
      proxySupplier = ProxyPushSupplierHelper.narrow (proxy);
      m_proxyReferenceSet = true;
   }


   public void connect ()
   {
      try
      {
         ProxySupplier ps = null;
         org.omg.CORBA.IntHolder idHolder = new org.omg.CORBA.IntHolder ();
         ps = consAdmin.obtain_notification_push_supplier (clientType, idHolder);
         proxySupplier = ProxyPushSupplierHelper.narrow (ps);
         applyQoSFilters (proxySupplier);

         if (consumer == null)
         {
            consumer = org.omg.CosEventComm.PushConsumerHelper.narrow (thisRef);
         }
         proxySupplier.connect_any_push_consumer (consumer);
      }
      catch (Exception ex)
      {
         System.err.println (this + ": failed to connect -- " + ex);
         ex.printStackTrace ();
         System.exit (-1);
      }
   }


   /**
    * Disconnects this client from the event channel.
    */
   public void disconnect ()
   {
      if (proxySupplier != null)
      {
         try
         {
            proxySupplier.disconnect_push_supplier ();
         }
         catch (Exception ex)
         {
            System.err.println (this + ": failed to disconnect -- " + ex);
            ex.printStackTrace ();
         }
      }
   }


   /**
    * Method called by the proxy supplier to send an event.
    * @param event  The untyped event.
    */
   public void push (org.omg.CORBA.Any event)
   {
      eventsReceived++;

      if (startTime == -1)
      {
         startTime = System.currentTimeMillis ();
      }

      currentTime = System.currentTimeMillis ();

      if (displayReceivedEvents)
      {
         displayAny (event);
      }

      reportIfNeeded (eventsReceived);
      delayIfNeeded (eventsReceived);
   }
}


// EOF
