/*************************************************************************
 *                           - COPYRIGHT NOTICE -                        *
 *                                                                       *
 * The information contained herein and which follows is proprietary     *
 * information and is the property of PrismTech Corporation.  This       *
 * information is not to be divulged, released, copied, reproduced, or   *
 * conveyed to unauthorized personnel, companies, or other institutions  *
 * without the direct and expressed approval in writing of PrismTech     *
 * Corporation.                                                          *
 *                                                                       *
 *                                                                       *
 *                           - DISCLAIMER -                              *
 * Neither PrismTech Limited nor PrismTech Corporation ("PrismTech")     *
 * are under any obligation to support the use of this code which is     *
 * provided without warranty of any kind. PrismTech shall (except as     *
 * otherwise prohibited by law) have no liability whatsoever in relation *
 * to its use.                                                           *
 *                                                                       *
 ************************************************************************/

/**
 * @author PrismTech
 * @version Version 1.0
 */

package com.prismt.cos.CosNotification.examples.Benchmark;

import java.io.*;

import org.omg.CosNotification.*;
import org.omg.CosNotifyComm.*;
import org.omg.CosNotifyChannelAdmin.*;
import org.omg.CosNotifyFilter.*;


/**
 * The benchmark sequence push consumer.
 */
public class BenchmarkSequenceConsumer extends BenchmarkConsumer
                                       implements SequencePushConsumerOperations,
                                                  java.io.Serializable
{
   private SequenceProxyPushSupplier proxySupplier = null;
   private SequencePushConsumer consumer = null;


   /**
    * Main method.  Creates an instance of the consumer and waits for events.
    * @param args  The command line arguments.
    */
   public static void main (String[] args)
   {
      BenchmarkSequenceConsumer cons = new BenchmarkSequenceConsumer ();
      cons.initialise (args);
      cons.connect ();

      System.out.println (cons + ": ready: press return to exit");
      BufferedReader br =
         new BufferedReader (new InputStreamReader (System.in));
      try
      {
         br.readLine ();
      }
      catch (IOException ioe)
      {
      }

      // If reporting, always display the final rate.
      if (cons.reportThreshold != 0)
      {
         if (cons.lastReportEventCount != cons.eventsReceived)
         {
            cons.report (cons.eventsReceived);
         }
      }
      
      cons.disconnect ();
      System.exit (0);
   }


   public BenchmarkSequenceConsumer ()
   {
      super ();
      isSupplier = false;
      clientType = ClientType.SEQUENCE_EVENT;
      setID ("BenchmarkSequenceConsumer");
   }


   public void setProxyReference (org.omg.CORBA.Object proxy)
   {
      proxySupplier = SequenceProxyPushSupplierHelper.narrow (proxy);
      m_proxyReferenceSet = true;
   }


   protected void prepareQoSProperties ()
   {
      Property[] qos = new Property[2];
      qos[0] = new Property ();
      qos[0].name = MaximumBatchSize.value;
      qos[0].value = orb.create_any ();
      qos[0].value.insert_long (batchSize);
      qos[1] = new Property ();
      qos[1].name = PacingInterval.value;
      qos[1].value = orb.create_any ();
      qos[1].value.insert_ulonglong ((long) pacingInterval);
      setQoSProperties (qos);
   }


   public void connect ()
   {
      try
      {
         ProxySupplier ps = null;
         org.omg.CORBA.IntHolder idHolder = new org.omg.CORBA.IntHolder ();
         ps = consAdmin.obtain_notification_push_supplier (clientType, idHolder);
         proxySupplier = SequenceProxyPushSupplierHelper.narrow (ps);
         applyQoSFilters (proxySupplier);
         
         if (consumer == null)
         {
            consumer = SequencePushConsumerHelper.narrow (thisRef);
         }
         proxySupplier.connect_sequence_push_consumer (consumer);
      }
      catch (Exception ex)
      {
         System.err.println (this + ": failed to connect -- " + ex);
         ex.printStackTrace ();
         System.exit (-1);
      }
   }


   /**
    * Disconnects this client from the event channel.
    */
   public void disconnect ()
   {
      if (proxySupplier != null)
      {
         try
         {
            proxySupplier.disconnect_sequence_push_supplier ();
         }
         catch (Exception ex)
         {
            System.err.println (this + ": failed to disconnect -- " + ex);
            ex.printStackTrace ();
         }
      }
   }


   /**
    * Method called by the proxy supplier to send events.
    * @param events  The array of structured event.
    */
   public void push_structured_events (StructuredEvent[] events)
   {
      eventsReceived += events.length;

      if (startTime == -1)
      {
         startTime = System.currentTimeMillis ();
      }

      currentTime = System.currentTimeMillis ();

      if (displayReceivedEvents)
      {
         for (int i = 0 ; i < events.length ; i++)
         {
            displayStructuredEvent (events[i]);
         }
      }

      reportIfNeeded (eventsReceived, events.length);
      delayIfNeeded (eventsReceived, events.length);
   }
}


// EOF
