package com.prismt.cos.CosNotification.examples.Security;

/*
 *                           - COPYRIGHT NOTICE -
 *
 * The information contained herein and which follows is proprietary
 * information and is the property of PrismTech Corporation.  This
 * information is not to be divulged, released, copied, reproduced, or
 * conveyed to unauthorized personnel, companies, or other institutions
 * without the direct and expressed approval in writing of PrismTech
 * Corporation.
 *
 */

/**
 * @author PrismTech
 * @version Version 1.0
 */

import org.omg.CosNotification.*;
import org.omg.CosNotifyComm.*;
import org.omg.CosNotifyChannelAdmin.*;

/**
 * Utility class that can be used to obtain an event channel
 */
public class ChannelUtil
{
  /* The resolveName of the NotificationService */
  private static String resolveName = "NotificationService";

   /**
    * Creates a new event channel.
    * @param orb the ORB to use
    * @return the event channel
    */
   public static EventChannel createChannel (org.omg.CORBA.ORB orb)
      throws org.omg.CORBA.ORBPackage.InvalidName
   {
      EventChannel channel = null;

      EventChannelFactory factory = null;

      /* In order to avoid problems with VisiBroker when registering the
       name NotificationService, rename it */

      if ((com.prismt.openfusion.Version.getORB().toUpperCase().startsWith("VISIBROKER-52")) ||
         (com.prismt.openfusion.Version.getORB().toUpperCase().startsWith("VISIBROKER-60")))
      {
         resolveName = "OFNotificationService";
      }
      
      /* End of specific code to deal with VisiBroker and registering
       NotificationService name problem*/

      /* resolve the Notification Service */
      System.out.println
         ("Resolving initial references to "+ resolveName);
       org.omg.CORBA.Object notRef =
           orb.resolve_initial_references (resolveName);
      System.out.println ("Success");

      // Create an event channel
      int channelNo=0;
      try
      {
         factory = EventChannelFactoryHelper.narrow (notRef);

         while( true )
         {
            try
            {
               channel = factory.get_event_channel (channelNo);
            }
            catch( org.omg.CORBA.NO_PERMISSION in )
            {
               System.out.println( "Permission not granted to retrieve a channel - from get_event_channel" );
               return null;
            }

            channelNo++;
         }
      }
      catch( org.omg.CORBA.NO_PERMISSION in )
      {
         System.out.println( "Permission not granted to retrieve a channel - from EventChannelFactoryHelper.narrow" );
         return null;
      }
      catch(ChannelNotFound cnf)
      {
         System.out.println( "Located an unused event channel: " + channelNo );

         // No such event channel, so create a new one
         Property[] adm = new Property[0];
         Property[] qos = new Property[2];
         org.omg.CORBA.IntHolder id = new org.omg.CORBA.IntHolder ();

         qos[0] = new Property ();
         qos[0].name = ConnectionReliability.value;
         qos[0].value = orb.create_any ();
         qos[0].value.insert_short (Persistent.value);

         qos[1] = new Property ();
         qos[1].name = EventReliability.value;
         qos[1].value = orb.create_any ();
         qos[1].value.insert_short (Persistent.value);

         try
         {
            channel = factory.create_channel (qos,adm, id);
         }
         catch ( org.omg.CORBA.NO_PERMISSION in )
         {
            System.out.println( "Permission not granted to create a channel - from create_channel" );
            return null;
         }
         catch(Exception e)
         {
            System.out.println ("Error: " + e);
            return null;
         }
      }

      return channel;
   }
}





