/*************************************************************************
 *                           - COPYRIGHT NOTICE -                        *
 *                                                                       *
 * The information contained herein and which follows is proprietary     *
 * information and is the property of PrismTech Corporation.  This       *
 * information is not to be divulged, released, copied, reproduced, or   *
 * conveyed to unauthorized personnel, companies, or other institutions  *
 * without the direct and expressed approval in writing of PrismTech     *
 * Corporation.                                                          *
 *                                                                       *
 *                                                                       *
 *                           - DISCLAIMER -                              *
 * Neither PrismTech Limited nor PrismTech Corporation ("PrismTech")     *
 * are under any obligation to support the use of this code which is     *
 * provided without warranty of any kind. PrismTech shall (except as     *
 * otherwise prohibited by law) have no liability whatsoever in relation *
 * to its use.                                                           *
 *                                                                       *
 ************************************************************************/

package com.prismt.cos.CosTrading.examples.Video;

import com.prismt.orb.*;

/**
 * An implementation of the <code>Film</code> interface.
 */

public class FilmImpl
   implements com.prismt.openfusion.examples.Video.FilmOperations
{
   private String filmTitle = null;
   private String filmCertificate = null;
   private VideoMachine machine = null;
   private org.omg.CORBA.Object object = null;

   /**
    * @param <code>title</code> The title of the film.
    * @param <code>certificate</code> The film certificate.
    * @param <code>vm</code> The video machine which shows the film.
    */
   public FilmImpl (String title, String certificate, VideoMachine vm)
   {
      filmTitle = title;
      filmCertificate = certificate;
      machine = vm;
   }

   /**
    * Broadcast the film (ie print a message)
    */
   public void show ()
   {
      machine.withdraw ();
      System.out.println ("Showing the film: " + title ());
      machine.export ();
   }

   /**
    * Return the film's title
    *
    * @return The title of the film.
    */
   public String title ()
   {
      return filmTitle;
   }

   /**
    * Return the film's certficate
    *
    * @return The certificate of the film.
    */
   public String certificate ()
   {
      return filmCertificate;
   }

   /**
    * Return the film's description. This is the combination of title and
    * certificate.
    *
    * @return The description of the film.
    */
   public String description ()
   {
      return (filmTitle + " " + filmCertificate);
   }
}
