/*************************************************************************
 *                           - COPYRIGHT NOTICE -                        *
 *                                                                       *
 * The information contained herein and which follows is proprietary     *
 * information and is the property of PrismTech Corporation.  This       *
 * information is not to be divulged, released, copied, reproduced, or   *
 * conveyed to unauthorized personnel, companies, or other institutions  *
 * without the direct and expressed approval in writing of PrismTech     *
 * Corporation.                                                          *
 *                                                                       *
 *                                                                       *
 *                           - DISCLAIMER -                              *
 * Neither PrismTech Limited nor PrismTech Corporation ("PrismTech")     *
 * are under any obligation to support the use of this code which is     *
 * provided without warranty of any kind. PrismTech shall (except as     *
 * otherwise prohibited by law) have no liability whatsoever in relation *
 * to its use.                                                           *
 *                                                                       *
 ************************************************************************/

package com.prismt.cos.CosTrading.examples.Video;

import org.omg.CosTrading.*;
import org.omg.CosTrading.LookupPackage.*;
import org.omg.CosTradingRepos.*;
import org.omg.CosTradingRepos.ServiceTypeRepositoryPackage.*;

import com.prismt.cos.CosTrading.examples.Video.*;
import com.prismt.orb.*;
import com.prismt.openfusion.examples.Video.*;

import org.omg.CORBA.TCKind;

/**
 * An implementation of a smart factory hidden behind a
 * <code>Lookup</code> interface. The factory is exported
 * to the trader as a proxy offer. When a query constraint
 * matches this proxy offer, the trader will invoke the
 * <code>query</code> operation of the <code>Factory</code>
 * object and return new instances of a film.
 * <p>
 * The advantage of this factory object is that objects are
 * discovered in a standard fashion using the trader service.
 * The clients of the trader are unaware of how the objects
 * are created. You can change the implementation of this
 * <code>Factory</code> completely without affecting any
 * of the clients.
 * <p>
 * This example reuses the IDL from the Video examples.
 * In order to compile the <code>Factory</code> class, you
 * will have to first compile the Video IDL and the java
 * classes generated by the IDL compiler. Alternatively, you
 * can easily modify this code to use a custom interface.
 */

public class ProgrammeFactory implements LookupOperations
{
   String title = null;

   /**
    * Bootstrap a <code>Factory</code>.
    *
    * @param <code>args</code> ORB arguments
    */
   public static void main(String [] args)
   {
      ProgrammeFactory factory = new ProgrammeFactory (args);
   }

   /**
    * The constructor exports the <code>Film</code> service type
    * to the trader repository. This service type has a single property
    * called <code>title</code>. Then, a proxy offer for this service
    * type will be created. This proxy offer is exported to the trader.
    *
    * @param <code>args</code> ORB arguments
    */
   public ProgrammeFactory(String [] args)
   {
      Lookup lookup = TradingUtil.getLookup ();

      org.omg.CORBA.ORB orb = ObjectAdapter.init (args);

      String serviceType  = "programme";
      Property [] property = new Property[0];
      boolean if_match_all = false;
      String recipe = "$*";
      Policy [] policies_to_pass_on = new Policy[0];

      Proxy proxy = lookup.proxy_if ();

      String id;

      try
      {
         id = proxy.export_proxy
         (
             lookup_if (),
             serviceType,
             property,
             if_match_all,
             recipe,
             policies_to_pass_on
         );
      }
      catch (Exception ex)
      {
         System.err.println ("Problem exporting proxy offer: " + ex);
         System.exit (1);
      }

      System.out.println ("Programme Factory is ready...");

      com.prismt.orb.ObjectAdapter.ready (); // dispatch incoming requests
   }

   /**
    * Indirected query.
    *
    * @param <code>type</code> The servivce type of the returned offers.
    * @param <code>constr</code> The constraint string passed on by the
    * trader. This constraint may be modified according to the recipe.
    * @param <code>pref</code> The preference string used to order returned
    * offers. This parameter is ignored.
    * @param <code>policies</code> Policies used to override default trader
    * query policies. This parameter is ignored.
    * @param <code>desired_props</code> Specifies which properties to return
    * with offers. This parameter is ignored.
    * @param <code>how_many</code> The maximum number of offers to be
    * returned. This parameter is ignored.
    * @param <code>offers</code> The sequence of returned offers. This
    * operation always returns a new instance of a <code>Film</code> object.
    * @param <code>offer_iter</code> A reference to an OfferIterator object
    * used to access offers not directly returned in the offer sequence.
    * This parameter is ignored by this operation.
    * @param <code>limit_applied</code> The limits applied by the trader
    * while processing the query. This parameter is ignored.
    * @exception IllegalServiceType If the specified service type name is
    * invalid. Not raised by this operation.
    * @exception UnknownServiceType If the specified service type has not
    * been registered. Not raised by this operation.
    * @exception IllegalConstraint If the constraint string is invalid. Not
    * raised by this operation.
    * @exception IllegalPreference If the preference string is invalid. Not
    * raised by this operation.
    * @exception IllegalPolicyName If an invalid policy name was specified.
    * Not raised by this operation.
    * @exception PolicyTypeMismatch If the type of a specified policy does
    * not match that of the specification. Not raised by this operation.
    * @exception InvalidPolicyValue If the value of a policy is invalid. Not
    * raised by this operation.
    * @exception IllegalPropertyName If a property name is illegal. Not
    * raised by this operation.
    * @exception DuplicatePropertyName If duplicate property names are
    * specified. Not raised by this operation.
    * @exception DuplicatePolicyName If duplicate policy names are specified.
    * Not raised by this operation.
    */
   public void query
   (
       String type,
       String constr,
       String pref,
       org.omg.CosTrading.Policy [] policies,
       SpecifiedProps desired_props,
       int how_many,
       OfferSeqHolder offers,
       OfferIteratorHolder offer_iter,
       PolicyNameSeqHolder limits_applied
   )
      throws IllegalServiceType,
             UnknownServiceType,
             IllegalConstraint,
             IllegalPreference,
             IllegalPolicyName,
             PolicyTypeMismatch,
             InvalidPolicyValue,
             IllegalPropertyName,
             DuplicatePropertyName,
             DuplicatePolicyName
   {
      int index1 = -1;
      int index2 = -1;
      System.out.println ("Programme Factory passed constraint: " + constr);
      index1 = constr.indexOf ("title==");
      index1 = index1 + 7; // move to the first quote mark
      if (index1 >= 0)
      {
         // "title==" appears in constraint
         index2 = constr.indexOf ("'", index1 + 7); // find second quote mark
         title = constr.substring (index1 + 1, index2);

         offers.value = new Offer[1];
         offers.value[0] = new Offer ();
         offers.value[0].properties = new Property[0];

         ProgrammeOperations pops = new ProgrammeOperations ()
         {
            public String title ()
            {
               return title;
            }

            public String description ()
            {
               return "Unknown";
            }

            public void show ()
            {
               System.out.println
               ("Showing from the programme factory: " + title);
            }
         };
         offers.value[0].reference = ObjectAdapter.createTransient (pops);

         limits_applied.value = new String[0];
      }
      else
      {
         // we can't create the programme
         offers.value = new Offer[0];
         limits_applied.value = new String[0];
         System.out.println
            ("Programme Factory unable to create a film using the constraint: "
             + constr);
      }

   }

   // All the operations required for <code>Lookup</code> objects.
   // These operations are not supported by this factory class.

   /**
    * Returns the Trader's Lookup interface
    *
    * @return The Trader's Lookup interface
    */
   public Lookup lookup_if ()
   {
      //org.omg.CORBA.Object ref = ObjectAdapter.getObject (this);
      org.omg.CORBA.Object ref = ObjectAdapter.createTransient (this);
      return (LookupHelper.narrow (ref));
   }

   /**
    * Returns null
    *
    * @return null
    */
   public Register register_if ()
   {
      return null;
   }

   /**
    * Returns null
    *
    * @return null
    */
   public Link link_if ()
   {
      return null;
   }

   /**
    * Returns null
    *
    * @return null
    */
   public Proxy proxy_if ()
   {
      return null;
   }

   /**
    * Returns null
    *
    * @return null
    */
   public Admin admin_if ()
   {
      return null;
   }

   /**
    * Returns false
    *
    * @return false
    */
   public boolean supports_modifiable_properties ()
   {
      return false;
   }

   /**
    * Returns false
    *
    * @return false
    */
   public boolean supports_dynamic_properties ()
   {
      return false;
   }

   /**
    * Returns false
    *
    * @return false
    */
   public boolean supports_proxy_offers ()
   {
      return false;
   }

   /**
    * Returns null
    *
    * @return null
    */
   public org.omg.CORBA.Object type_repos ()
   {
      return null;
   }

   /**
    * Returns 0
    *
    * @return 0
    */
   public int def_search_card ()
   {
      return 0;
   }

   /**
    * Returns 0
    *
    * @return 0
    */
   public int max_search_card ()
   {
      return 0;
   }

   /**
    * Returns 0
    *
    * @return 0
    */
   public int def_match_card ()
   {
      return 0;
   }

   /**
    * Returns 0
    *
    * @return 0
    */
   public int max_match_card ()
   {
      return 0;
   }

   /**
    * Returns 0
    *
    * @return 0
    */
   public int def_return_card ()
   {
      return 0;
   }

   /**
    * Returns 0
    *
    * @return 0
    */
   public int max_return_card ()
   {
      return 0;
   }

   /**
    * Returns 0
    *
    * @return 0
    */
   public int max_list ()
   {
      return 0;
   }

   /**
    * Returns 0
    *
    * @return 0
    */
   public int def_hop_count ()
   {
      return 0;
   }

   /**
    * Returns 0
    *
    * @return 0
    */
   public int max_hop_count ()
   {
      return 0;
   }

   /**
    * Returns FollowOption.local_only
    *
    * @return FollowOption.local_only
    */
   public FollowOption def_follow_policy ()
   {
      return FollowOption.local_only;
   }

   /**
    * Returns FollowOption.local_only
    *
    * @return FollowOption.local_only
    */
   public FollowOption max_follow_policy ()
   {
      return FollowOption.local_only;
   }
}
