/*************************************************************************
 *                           - COPYRIGHT NOTICE -                        *
 *                                                                       *
 * The information contained herein and which follows is proprietary     *
 * information and is the property of PrismTech Corporation.  This       *
 * information is not to be divulged, released, copied, reproduced, or   *
 * conveyed to unauthorized personnel, companies, or other institutions  *
 * without the direct and expressed approval in writing of PrismTech     *
 * Corporation.                                                          *
 *                                                                       *
 *                                                                       *
 *                           - DISCLAIMER -                              *
 * Neither PrismTech Limited nor PrismTech Corporation ("PrismTech")     *
 * are under any obligation to support the use of this code which is     *
 * provided without warranty of any kind. PrismTech shall (except as     *
 * otherwise prohibited by law) have no liability whatsoever in relation *
 * to its use.                                                           *
 *                                                                       *
 ************************************************************************/


package com.prismt.cos.CosTrading.examples.Video;

import org.omg.CosTrading.*;
import org.omg.CosTradingRepos.*;
import org.omg.CosTradingRepos.ServiceTypeRepositoryPackage.*;
import com.prismt.orb.ORBAdapter;
import com.prismt.orb.*;


/**
 * This class is responsible for adding the Film and Sports Event service
 * types and their Programme supertype to the trading service.
 */

public class TVCompany
{
   private org.omg.CORBA.ORB orb = null;

   /**
    * Create a new TVCompany object.
    *
    * @param <code>args</code> ORB arguments
    */
   public static void main (String [] args)
   {
      TVCompany tvc = new TVCompany (args);
   }

   /**
    * Obtain a reference to the Service Type Repository and then call its
    * <code>add_type</code> method to add the service types to the trader. The
    * service types that are added to the trader are IDL:Video/Programme:1.0,
    * IDL:Video/Film:1.0 and IDL:Video/SportsEvent:1.0.
    *
    * @param <code>args</code> ORB arguments
    */
   public TVCompany (String [] args)
   {
      ServiceTypeRepository repos;
      String interfaceType;
      String serviceType;
      PropStruct [] props;
      String [] supers;

      //In order to make examples easier to run, for Orbacus-40 set a POAName
      if (com.prismt.openfusion.Version.getORB().
            toUpperCase().startsWith("ORBACUS-4"))
      {
         ObjectAdapter.setPOAName("OpenFusion.TradingTVCompany");
      }

      orb = ORBAdapter.init (args);

      repos = TradingUtil.getTypeRepos ();

      // add programme type

      serviceType = "programme";
      interfaceType
         = "IDL:prismt.com/openfusion/examples/Video/Programme:1.0";

      props = new PropStruct[1];
      props[0] = new PropStruct ();
      props[0].name = "title";
      props[0].mode = PropertyMode.PROP_NORMAL;
      props[0].value_type =
         orb.get_primitive_tc (org.omg.CORBA.TCKind.tk_string);

      // this type has no supertypes
      supers = new String[0];
      try
      {
         repos.add_type (serviceType, interfaceType, props, supers);
         System.out.println ("Service type " + serviceType + " successfully added");
      }
      catch (Exception ex)
      {
         System.err.println ("Problem adding service type: " + ex);
      }

      /* add sports event type */

      serviceType = "sportsEvent";
      interfaceType
         = "IDL:prismt.com/openfusion/examples/Video/SportsEvent:1.0";

      props = new PropStruct[1];
      props[0] = new PropStruct ();
      props[0].name = "sport";
      props[0].mode = PropertyMode.PROP_NORMAL;
      props[0].value_type =
         orb.get_primitive_tc (org.omg.CORBA.TCKind.tk_string);
      supers = new String[1];
      supers[0] = "programme";

      try
      {
         repos.add_type (serviceType, interfaceType, props, supers);
         System.out.println ("Service type " + serviceType + " successfully added");
      }
      catch (Exception ex)
      {
         System.err.println ("Problem adding service type: " + ex);
      }

      // add film type

      serviceType = "film";
      interfaceType = "IDL:prismt.com/openfusion/examples/Video/Film:1.0";
      props = new PropStruct[1];
      props[0] = new PropStruct ();
      props[0].name = "certificate";
      props[0].mode = PropertyMode.PROP_NORMAL;
      props[0].value_type =
         orb.get_primitive_tc (org.omg.CORBA.TCKind.tk_string);
      supers = new String[1];
      supers[0] = "programme";
      try
      {
         repos.add_type (serviceType, interfaceType, props, supers);
         System.out.println ("Service type " + serviceType + " successfully added");
      }
      catch (Exception ex)
      {
         System.err.println ("Problem adding service type: " + ex);
      }

      //Orbacus shutdown fix
      System.exit (0);
   }
}
