/*************************************************************************
 *                           - COPYRIGHT NOTICE -                        *
 *                                                                       *
 * The information contained herein and which follows is proprietary     *
 * information and is the property of PrismTech Corporation.  This       *
 * information is not to be divulged, released, copied, reproduced, or   *
 * conveyed to unauthorized personnel, companies, or other institutions  *
 * without the direct and expressed approval in writing of PrismTech     *
 * Corporation.                                                          *
 *                                                                       *
 *                                                                       *
 *                           - DISCLAIMER -                              *
 * Neither PrismTech Limited nor PrismTech Corporation ("PrismTech")     *
 * are under any obligation to support the use of this code which is     *
 * provided without warranty of any kind. PrismTech shall (except as     *
 * otherwise prohibited by law) have no liability whatsoever in relation *
 * to its use.                                                           *
 *                                                                       *
 ************************************************************************/

package com.prismt.cos.CosTrading.examples.Video;

import java.util.Random;

import org.omg.CosTrading.*;
import org.omg.CosTrading.LookupPackage.*;
import com.prismt.orb.ORBAdapter;
import com.prismt.orb.*;
import com.prismt.openfusion.examples.Video.*;

/**
 * This class is a representation of a television set that can request a video
 * feed.
 */

public class TVSet
{
   private static org.omg.CORBA.ORB orb = null;
   private Lookup lookup;
   Random rand;

   /**
    * Create a new TVSet object.
    *
    * @param <code>args</code> ORB arguments
    */
   public static void main (String [] args)
   {
      //In order to make examples easier to run, for Orbacus-40 set a POAName
      if (com.prismt.openfusion.Version.getORB().
            toUpperCase().startsWith("ORBACUS-4"))
      {
         ObjectAdapter.setPOAName("OpenFusion.TradingTVSet");
      }

      orb = ORBAdapter.init (args);
      TVSet tv = new TVSet ();
   }

   /**
    * The constructor randomly chooses a programme. It then calls the
    * <code>query</code> method to obtain an offer of a video feed of that
    * programme. The <code>show</code> method of the Programme is called
    * when a offer of service is obtained.
    */
   public TVSet ()
   {
      // pick a title to ask the trader for

      rand = new Random ();
      int randTitle  = Math.abs (rand.nextInt () % (allProgrammes.length));
      String title = allProgrammes[(randTitle)% allProgrammes.length];

      System.out.println ("TVSet is requesting the programme " +  title);

      lookup = TradingUtil.getLookup ();

      Offer offer = query (title);

      if (offer != null)
      {
         Programme prog = ProgrammeHelper.narrow (offer.reference);
         System.out.println ("Programme beginning...");
         prog.show ();
         System.out.println ("Programme ended");
      }
      else
      {
         System.out.println ("Programme not available");
      }

      //Orbacus shutdown fix
      System.exit (0);
   }

   /**
    * This method creates and performs a query to ask the trader where to
    * obtain a video feed of the specified programme.
    *
    * @param <code>title</code> The title of the programme.
    * @return The matched <code>Offer</code> for that programme title or
    * null if no match was found
    */
   public Offer query (String title)
   {
      String serviceType = "programme";
      String constraint = "title=='" + title + "'";
      String preferences = "";
      Policy [] policies = new Policy[0];
      SpecifiedProps properties = new SpecifiedProps ();
      int how_many = 3;
      OfferIteratorHolder offerItr = new OfferIteratorHolder ();
      OfferSeqHolder offerSeq = new OfferSeqHolder ();
      PolicyNameSeqHolder limits = new PolicyNameSeqHolder ();

      properties.__default (HowManyProps.none);

      try
      {
         lookup.query
         (
            serviceType,
            constraint,
            preferences,
            policies,
            properties,
            how_many,
            offerSeq,
            offerItr,
            limits
         );
      }
      catch (Exception ex)
      {
         ex.printStackTrace (System.err);
         System.err.println ("Problem querying trader: " + ex);
      }

      Offer offer = null;

      if (offerSeq.value.length > 0)
      {
         offer = offerSeq.value[0];
      }

      return offer;
   }

   static String allProgrammes [] =
   {
      "ET",
      "Goldfinger",
      "Gone with the Wind",
      "Ivanhoe",
      "Jurassic Park",
      "Mary Poppins",
      "Pulp Fiction",
      "Star Wars",
      "Titanic",
      "The Temple of Doom",
      "The Untouchables",
      "Seven",
      "Reservoir Dogs",
      "Superbowl",
      "Wimbledon Singles Final",
      "Manchester United v. Arsenal",
      "US Open Final",
      "FA Cup Final",
      "NY Giants v. Arizona Cardinals"
   };
}
