/*************************************************************************
 *                           - COPYRIGHT NOTICE -                        *
 *                                                                       *
 * The information contained herein and which follows is proprietary     *
 * information and is the property of PrismTech Corporation.  This       *
 * information is not to be divulged, released, copied, reproduced, or   *
 * conveyed to unauthorized personnel, companies, or other institutions  *
 * without the direct and expressed approval in writing of PrismTech     *
 * Corporation.                                                          *
 *                                                                       *
 *                                                                       *
 *                           - DISCLAIMER -                              *
 * Neither PrismTech Limited nor PrismTech Corporation ("PrismTech")     *
 * are under any obligation to support the use of this code which is     *
 * provided without warranty of any kind. PrismTech shall (except as     *
 * otherwise prohibited by law) have no liability whatsoever in relation *
 * to its use.                                                           *
 *                                                                       *
 ************************************************************************/

package com.prismt.cos.CosTrading.examples.Video;

import java.util.Random;
import org.omg.CosTrading.*;
import com.prismt.orb.*;

/**
 * This class is a representation of a server that supplies video feeds.
 */

public class VideoMachine
{
   private static org.omg.CORBA.ORB orb = null;

   private FilmImpl [] films;
   private SportsEventImpl [] sports;
   private String [] offerIds;
   Register register;

   /**
    * The main method initialises the ORB, creates a new Video Machine object,
    * and calls ObjectAdapter.ready () to establish the VideoMachine as a
    * server.
    *
    * @param <code>args</code> ORB arguments
    */
   public static void main (String [] args)
   {
      try
      {
         //In order to make examples easier to run, for Orbacus-40 set a POAName
         if (com.prismt.openfusion.Version.getORB().
               toUpperCase().startsWith("ORBACUS-4"))
         {
            ObjectAdapter.setPOAName("OpenFusion.TradingVideoMachine");
         }
         orb = ObjectAdapter.init (args);
      }
      catch (Exception ex)
      {
         System.err.println ("Failed to initialise server: " + ex);
         System.exit (0);
      }

      VideoMachine vm = new VideoMachine (args);

      System.out.println ("Entering server event loop");

      // non blocking mode so that user can hit Enter to exit program
      //
      ObjectAdapter.ready (false);

      try
      {
         System.out.println( "Press Enter to exit program...");
         int nextByte = System.in.read();
      }
      catch (java.io.IOException ioEx)
      {
      }

      // shutdown ObjectAdapter and deactivate all active objects
      //
      ObjectAdapter.shutdown ();
      System.exit (0);
   }

   /**
    * The constructor randomly chooses a number of television programmes for
    * which it can provide video feeds, and then calls the <code>export</code>
    * method to export offers of these video feeds.
    *
    * @param <code>args</code> ORB arguments
    */
   public VideoMachine (String [] args)
   {
      int randTitle;
      int randNumber;
      Random rand;
      FilmImpl film;
      SportsEventImpl sport;

      rand = new Random ();

      System.out.println ("Video machine can play: ");

      // create some films for this video machine to be able to play

      randTitle  = 1 + Math.abs (rand.nextInt () % (allFilms.length - 1));
      randNumber = 5 + Math.abs (rand.nextInt () % (allFilms.length - 5));

      films = new FilmImpl [randNumber];

      for (int i = 0 ; i < films.length ; i++)
      {
         film = new FilmImpl
         (
            allFilms [(randTitle + i) % allFilms.length][0],
            allFilms [(randTitle + i) % allFilms.length][1],
            this
         );
         ObjectAdapter.createTransient (film);
         films [i] = film;

         System.out.println ("   " + films [i].title() +
                             " " + films [i].certificate());
      }

      // create some sports events for this video machine to be able to play

      randTitle  = 1 + Math.abs (rand.nextInt () % (allSports.length - 1));
      randNumber = 3 + Math.abs (rand.nextInt () % (allSports.length - 3));

      sports = new SportsEventImpl[randNumber];

      for (int i = 0 ; i < sports.length ; i++)
      {
         sport = new SportsEventImpl
         (
            allSports [(randTitle + i) % allSports.length][0],
            allSports [(randTitle + i) % allSports.length][1],
            this
         );
         ObjectAdapter.createTransient (sport);
         sports [i] = sport;

         System.out.println ("   " + sports [i].title() +
                             " " + sports [i].sport());
      }

      // now add service types to the trading service
      register = TradingUtil.getRegister ();

      export ();
   }

   /**
    * This method withdraws all the previously exported offers of video feeds
    * from the trading service.
    */
   public void withdraw ()
   {
      // remove all offers from the trading service
      System.out.println ("Withdrawing all films from the trading service");
      try
      {
         for (int i = 0 ; i < offerIds.length ; i++)
         {
            register.withdraw (offerIds[i]);
         }
         offerIds = new String[0];
      }
      catch (Exception ex)
      {
         System.err.println ("Problem withdrawing offer: " + ex);
      }
      System.out.println ("Ready to show a film");
   }

   /**
    * This method exports a number of offers of video feeds to the
    * trading service.
    */
   public void export ()
   {
      String serviceType;
      Property [] prop;
      org.omg.CORBA.Any any;
      int i;
      int j;

      // add all offers into the trading service
      System.out.println ("Exporting available films to the trading service");
      offerIds = new String [films.length + sports.length];
      for (i = 0 ; i < films.length ; i++)
      {
         serviceType = "film";
         prop = new Property[2];
         prop[0] = new Property ();
         prop[0].name = "title";
         any = orb.create_any ();
         any.insert_string (films[i].title());
         prop[0].value = any;
         prop[1] = new Property ();
         prop[1].name = "certificate";
         any = orb.create_any ();
         any.insert_string (films[i].certificate());
         prop[1].value = any;
         try
         {
            offerIds[i] = register.export
               (ObjectAdapter.getObject (films[i]), serviceType, prop);
         }
         catch (Exception ex)
         {
            System.err.println ("Problem exporting offer: " + ex);
         }
      }
      for (i = 0 ; i < sports.length ; i++)
      {
         j = i + films.length;

         serviceType = "sportsEvent";
         prop = new Property[2];
         prop[0] = new Property ();
         prop[0].name = "title";
         any = orb.create_any ();
         any.insert_string (sports[i].title());
         prop[0].value = any;
         prop[1] = new Property ();
         prop[1].name = "sport";
         any = orb.create_any ();
         any.insert_string (sports[i].sport());
         prop[1].value = any;
         try
         {
            offerIds[j] = register.export
               (ObjectAdapter.getObject (sports[i]), serviceType, prop);
         }
         catch (Exception ex)
         {
            System.err.println ("Problem exporting offer: " + ex);
         }
      }

      System.out.println ("Waiting to be asked to show a programme....");
   }

   static String allFilms [] [] =
   {
      {"ET", "PG"},
      {"Goldfinger", "15"},
      {"Gone with the Wind", "U"},
      {"Jurassic Park", "12"},
      {"Mary Poppins", "U"},
      {"Pulp Fiction", "18"},
      {"Star Wars", "U"},
      {"Titanic", "12"},
      {"The Temple of Doom", "PG"},
      {"The Untouchables", "18"},
      {"Seven", "18"},
      {"Reservoir Dogs", "18"},
      {"The Matrix", "18"}
   };

   static String allSports [] [] =
   {
      {"Superbowl", "American Football"},
      {"Wimbledon Singles Final", "Tennis"},
      {"Manchester United v. Arsenal", "Soccer"},
      {"US Open Final", "Tennis"},
      {"FA Cup Final", "Soccer"},
      {"NY Giants v. Arizona Cardinals", "American Football"}
   };
}
