package org.jacorb.imrutility.imr.persistence;

 
import java.io.*;
import java.util.HashMap;

import org.jacorb.util.Debug;
import org.jacorb.util.Environment;

import org.apache.avalon.framework.logger.*;

/**
 * This class is responsible for the persistence of data added to and
 * removed from the JacORB Implementation Repository. 
 * @see org.jacorb.imrutility.imr.RepositoryImpl where the persistence
 * mechanism is file.  The persistence mechanism will be configurable
 */ 
public class ActFilePersistence
{  
    /**
     * Logs output
     */
    protected Logger logger = Debug.getNamedLogger("jacorb.activator");    
    
    /**
     * This method reads data from persistence and populates its own 
     * internal tables with the data retrieved.
     */
    public ActivatorData read()
    {
        ActivatorData actData = null;
        
        String dataFile = Environment.getProperty("jacorb.imr.act_data_file");
        
        if (dataFile != null)
        {
            File f = new File(dataFile);
            
            try
            {
                if (f.exists())
                {
                    ObjectInputStream ois =
                        new ObjectInputStream(new FileInputStream(f));
                
                    actData = (ActivatorData) ois.readObject();            
                }
            }
            catch (IOException ioe)
            {
                if (logger.isDebugEnabled())
                {
                    logger.warn
                        ("WARNING: Unable to read Activator data file " 
                            + dataFile +"! ");
                }            
            }
            catch (ClassNotFoundException cnfe)
            {
                if (logger.isDebugEnabled())
                {
                    logger.warn
                        ("WARNING: Activator data file " + dataFile  
                            + " does not contain valid object" +"! ");                    
                }
            }
        }
        else
        {
            if (logger.isDebugEnabled())
            {
                logger.warn("WARNING: Activator data file name not configured! ");                     
            }            
        }                        
        
        if (actData == null)
        {
            actData = new ActivatorData();
        }
        
        return actData;
    }
    
    /**
     * This method writes the data from memory to persistence
     */
    public void write(ActivatorData actData)
    {  
        File f;
        String dataFile = Environment.getProperty("jacorb.imr.act_data_file");
        
        if (dataFile == null)
        {
            if (logger.isDebugEnabled())
            {
                logger.warn
                ( 
                    "WARNING: Activator data file name not configured"
                    + " creating \"actdata.dat\" in current directory, " 
                );
                    
                dataFile = "actdata.dat";
            }            
        }

        f = new File(dataFile);        

        write(f, actData);
    }

    /**
     * This method is used where a backup facility exists within the persistence
     * mechanism.  With file persistence a backup file is also created at shutdown
     * of the Activator to ensure  that data is not lost.
     */
    public void backup(ActivatorData actData)
    {
        File f;
        String dataFile = Environment.getProperty("jacorb.imr.act_backup_file");
        boolean write = true;
        
        if (dataFile != null)
        {
            f = new File(dataFile);        
        
            write(f, actData);  
        }
    }
    
    /**
     * Internal write method that is used whether data is being written to the
     * main file or the backup file
     * @param <code>file</code> the name of the file to which the data is to 
     * be written
     */
    private void write(File file, ActivatorData actData)
    { 
        try
        {
            ObjectOutputStream oos = 
                new ObjectOutputStream(new FileOutputStream(file));
            oos.writeObject(actData);     
            oos.flush();
            oos.close();
        }
        catch (IOException ioe)
        {
            if (logger.isDebugEnabled())
            {
                logger.warn
                (
                    "Unable to write Activator data to file " + file.getName()
                    + " :: " + ioe.getMessage() 
                );            
            }
        }
    }
}
   
