package org.jacorb.test.imrutility.imr;

import junit.framework.*;

import org.jacorb.imrutility.imr.*;
import org.jacorb.util.Environment;

import java.io.BufferedReader;
import java.io.IOException;
import java.io.FileReader;
import java.util.Properties;

public class AutoActivationTest extends PersistentServerTestCase
{
    private String serverIOR;
    private final String ior = "server.ior";
    private final String home = System.getProperty("user.home");
    private final String separator = System.getProperty("file.separator");
    private Process proc;
    private RepositoryAdmin rep;
    private Activator act;
        
    public AutoActivationTest(String name, PersistentServerSetup setup)
    {
        super(name, setup);
    }

    public static Test suite()
    {
        TestSuite suite = new TestSuite ("Auto Activation Test");
                       
        PersistentServerSetup setup =
            new PersistentServerSetup
            ( 
                suite,
                "org.jacorb.test.imrutility.imr.Server"
            );
 
        suite.addTest (new AutoActivationTest ("testAutoActivation", setup));
        
        // Return the setup, not the suite!
        return setup;
    }
 
    protected void tearDown()
    {
        System.err.println("********** tearDown for testAutoActivation");
        try
        {
            if (proc != null)
            {
                proc.destroy();
            }
         
            if (rep != null)
            {       
                rep.shutdown(false);
            }
        
            TestUtility.removeIMRFiles();
            TestUtility.removeFile(serverIOR);
            TestUtility.destroyImR();
            TestUtility.removeActivatorFiles();
            TestUtility.stopActivator();
            Thread.sleep(1000);
        }
        catch (Exception ex)
        {
            /* don't do anything */
        }                            
    }
       
    public void testAutoActivation()
    {
        Process proc2 = null;
        ImplInfo [] infos = null;
        StringBuffer command = null;
        ServerInfo [] sinfos = null;
        POAInfo [] poas = null;
        TestServer test = null;
        ServerName sname = null;
        long originalID = 0;
        String [] args = null;
        String [] actArgs = null;
        BufferedReader br = null;
        String srvIOR = null;
        CommandInfo [] cinfos = null;
                                                            
        try
        {        
            /* Delete all files so no old data hanging around*/
            TestUtility.removeIMRFiles();
            
            /* Wait for files to be removed*/
            Thread.sleep(1000);
        }
        catch (Exception ex)
        {
            /* Don't do anything as this is not a problem*/
        }

        //Start the IMR        
        try
        {                     
            args = new String[2];
            args[0] = "-p";
            args[1] = "23451";            
            
            rep = TestUtility.startImR(args, null);
            
            assertTrue("Failed to get ImR", rep != null);
        }
        catch(Exception ex)
        {
            fail("Exception getting ImR" + ex.getMessage());
        }

        /* get command to start server, start then stop server - gets IOR file*/        
        try
        {
            command = setup.getStartCommand();
            command.append(" -i ");
            serverIOR = home + separator + ior;
            command.append(serverIOR);
                        
            proc = TestUtility.startServer(command);
            
            proc.destroy();
        }
        catch (IOException ioe)
        {
            fail("Got IOException starting server " + ioe.getMessage());
        }
        catch (Exception ex)
        {
            fail("Got Exception starting server " + ex.getMessage());
        }
                
        //Stop and restart the IMR        
        try
        {                         
            TestUtility.destroyImR();

            TestUtility.removeFile
                (Environment.getProperty("jacorb.imr.ior_file"));                     
                 
            /* restart IMR on same port */   
            rep = TestUtility.startImR(args, null);
                        
            assertTrue("Failed to get ImR 2nd time", rep != null);
                        
            infos = rep.listAllImpls();
        
            assertTrue("Implementation not registered", infos.length == 1);
                    
        }
        catch(Exception ex)
        {
            fail("Exception getting ImR" + ex.getMessage());
        } 

        try
        {        
            br = new BufferedReader(new FileReader(serverIOR));

            // get object reference from command-line argument file
            org.omg.CORBA.Object obj =
                TestUtility.getImRORB().string_to_object(br.readLine());

            br.close();

            //narrow to correct interface
            test = TestServerHelper.narrow(obj);
            
            test.ping();
                        
            fail("Expected an exception pinging the server");
        }
        catch (Exception ex)
        {
            // do nothing - Exception expected cannot 
            // start server as no Activator registered 
        }

        sname = new ServerName();
        sname.impl("test_imr");
                
        try
        {   
            actArgs = new String [0];     
            rep.removeImpl(sname, null);

            infos = rep.listAllImpls();
            
            assertTrue("Impl not removed", infos.length == 0);
                                    
            act = TestUtility.startActivator(actArgs, null);
                        
            assertTrue("Activator not found", act != null);
            
            cinfos = act.listCommands();
            
            assertTrue("Commands registered", cinfos.length == 0);            
            
            TestUtility.stopActivator();
            
            TestUtility.removeActivatorFiles();              
                      
            ImplInfo info = 
                new ImplInfo
                (
                    sname, 
                    false, 
                    LoadBalancingPolicy.from_int(0),
                    ActivationMode.from_int(3)
                );            
            
            try
            {            
                rep.addImpl(info, act);
                fail("Expected an exception as the Activator is not running");
            }
            catch (Exception ex)
            {
                // DO nothing exception expected as Activator not running
            }
                                               
            act = TestUtility.startActivator(actArgs, null);
            
            rep.addImpl(info, act);
                        
            /* Tests activator running but no command registered*/
            try
            {
                test.ping();
                fail("Expected an exception pinging server");
            }
            catch (Exception ex)
            {
                // do nothing - exception expected
                // cannot start server as start command not registered
            }           
            
            CommandInfo cinfo = 
                new CommandInfo
                    (sname, "start me now");
                                
            act.addCommand(cinfo);
            
            cinfos = act.listCommands();
            
            assertTrue("Command not registered", cinfos.length == 1);
            
            /* Tests Activator running with invalid command*/ 
            try
            {
                test.ping();
                fail("Expected an exception pinging server");
            }
            catch (Exception ex)
            {                
                // do nothing - exception expected
                // cannot start server as start up command is invalid
            }
                       
            act.removeCommand(sname);

            cinfos = act.listCommands();
            
            assertTrue("Command not removed", cinfos.length == 0);
            
            cinfo = 
                new CommandInfo
                    (sname, command.toString());
                                
            act.addCommand(cinfo);
            
            cinfos = act.listCommands();
                    
            /* Tests Activator running with valid command*/                                                      
            try
            {
                test.ping();                                
            }
            catch (Exception ex)       
            {                  
                fail("Unexpected exception pinging Server");              
            }           
        }
        catch (Exception ex)
        {
            fail("Unexpected exception " + ex.getMessage());
        }
        
        test.shutdown();        
    }
}

