package org.jacorb.test.imrutility.imr;

import junit.extensions.*;
import junit.framework.*;

import org.jacorb.test.common.ORBSetup;

import org.omg.CORBA.ORB;

import java.util.Iterator;
import java.util.Properties;

public class PersistentServerSetup extends TestSetup
{
    protected String                     servantName;
    protected Process                    serverProcess;
    protected org.omg.CORBA.Object       serverObject;

    private Properties serverOrbProperties = null;

    private String [] args;
        
    private StringBuffer startCommand;

    public PersistentServerSetup(Test test, String servantName)
    {
        super(test);
        this.servantName = servantName;
    }

    public PersistentServerSetup
    (
        Test test,
        String servantName,
        Properties serverOrbProperties,
        String [] args
    )
    {
        this(test, servantName);

        this.serverOrbProperties = serverOrbProperties;
        this.args = args;
    }

    public void setUp() throws Exception
    {
        /* Initialise the Environment*/
        org.jacorb.util.Environment.init();                                                     
    }

    public void tearDown() throws Exception
    {
    }

    public String getTestServerMain()
    {
        return "org.jacorb.test.common.TestServer";
    }

    public StringBuffer getStartCommand()
    {
        startCommand = new StringBuffer("jaco -Djacorb.implname=");
        startCommand.append("test_imr");
        startCommand.append(' ');
        startCommand.append(propsToCommandLineArgs(serverOrbProperties));
        startCommand.append(" -classpath ");
        startCommand.append(System.getProperty ("java.class.path"));
        startCommand.append(' ');
        startCommand.append(servantName);
        
        if (args != null)
        {
            for (int i = 0; i < args.length; i++)
            {
                startCommand.append(' ');
                startCommand.append(args[i]);   
            }
        }                
        
        return startCommand;
    }
    
    private static String propsToCommandLineArgs( Properties props )
    {
        if( props == null )
        {
            return "";
        }

        StringBuffer sb = new StringBuffer();

        for( Iterator keyIterator = props.keySet().iterator();
             keyIterator.hasNext();
            )
        {
            String key = (String) keyIterator.next();
            sb.append( "-D" );
            sb.append( key );
            sb.append( '=' );

            String value = props.getProperty( key );
            if( value == null )
            {
                value = "";
            }

            sb.append( value );

            if( keyIterator.hasNext() )
            {
                sb.append( ' ' );
            }
        }

        return sb.toString();
    }
}
