package org.jacorb.test.imrutility.imr;

import java.io.*;

import org.omg.CORBA.*;
import org.omg.PortableServer.*;

import org.jacorb.util.Debug;
import org.jacorb.util.Environment;

import org.apache.avalon.framework.logger.*;

public class Server extends TestServerPOA
{
    private static ORB orb;
    
    private static String iorfile;

    private Logger logger;
    
    public static void main(String[] args)
    {
        String value = null;
        
        if( args.length == 0 )
        {
            System.out.println(
                "Usage: jaco Server <ior_file>");
            System.exit( 1 );
        }
        
        if (args.length > 0)
        {
            for (int i = 0; i < args.length; i++)
            {
                if (i < args.length - 1)
                {
                    if (args[i].equals("-u"))
                    {                       
                        value = args[++i];
                        Environment.setProperty("jacorb.imr.udp_port", value);                        
                    }
                    else if (args[i].equals("-h"))
                    {
                        Environment.setProperty("jacorb.imr.heartbeat", args[++i]);
                    } 
                    else if (args[i].equals("-i"))
                    {
                        iorfile = args[++i];    
                    }                   
                }
            }
        }

        try
        {
            //instantiate implementation
            Server ti = new Server(args);
        }
        catch (Exception ex)
        {
            System.exit(1);            
        }
    }
    
    public Server(String[] args)
       throws Exception
    {   
        
        logger = Debug.getNamedLogger("jacorb.imr.test");
                
        System.setProperty( "jacorb.implname", "test_imr" );
        System.setProperty( "jacorb.use_imr", "on" );
        String value;        
        
        if (iorfile == null)
        {
            throw new Exception("Server IOR file not provided");            
        }
        
        try
        {
            //init ORB
            orb = ORB.init( args, null );

            //get root POA
            org.omg.PortableServer.POA root_poa =
            org.omg.PortableServer.POAHelper.narrow(
                    orb.resolve_initial_references("RootPOA"));

            //create necessary policies
            org.omg.CORBA.Policy[] policies = new org.omg.CORBA.Policy[2];

            policies[0] = root_poa.create_lifespan_policy(LifespanPolicyValue.PERSISTENT);

            policies[1] = root_poa.create_id_assignment_policy(IdAssignmentPolicyValue.USER_ID);

            //create user POA with these policies
            POA test_poa = root_poa.create_POA( "TestIMRPOA",
                                                root_poa.the_POAManager(),
                                                policies );

            //make POA accept requests
            test_poa.the_POAManager().activate();

            //destroy policies
            for (int i=0; i<policies.length; i++)
            {
                policies[i].destroy();
            }
            
            //create object id
            byte[] id = "imr_test".getBytes();

            //activate object
            test_poa.activate_object_with_id( id, this );

            logger.debug("back from activating object with id");

            // create the object reference
            org.omg.CORBA.Object obj =
                test_poa.servant_to_reference( this );

            //make POA accept requests
            root_poa.the_POAManager().activate();

            logger.debug("Going to write the ior to " + iorfile);

            PrintWriter pw =
                new PrintWriter( new FileWriter(iorfile));

            // print stringified object reference to file
            pw.println( orb.object_to_string( obj ));

            pw.flush();
            pw.close();

            orb.run();
        }
        catch( Exception e )
        {
            logger.debug("Exception", e);
        }
    }


    public Server()
    {
    }
    
    public void ping()
    {
        logger.debug("Got pinged by a client");
    }
    
    public void shutdown()
    {
        orb.shutdown(false);       
    }
}
