package org.jacorb.test.imrutility.imr;

import junit.framework.*;

import org.jacorb.imrutility.imr.*;
import org.jacorb.util.Environment;

import java.io.*;

/**
 * This class tests the location of an ImR using UDP
 */
public class UDPTest extends PersistentServerTestCase
{
    private String serverIOR;
    private final String ior = "server.ior";
    private final String home = System.getProperty("user.home");
    private final String separator = System.getProperty("file.separator");
    private Process proc;
    private RepositoryAdmin rep;

        
    public UDPTest(String name, PersistentServerSetup setup)
    {
        super(name, setup);
    }

    public static Test suite()
    {
        TestSuite suite = new TestSuite ("UDP Test");
        String [] args = new String [2];
        args[0] = "-u";
        args[1] = "23456";
                     
        PersistentServerSetup setup =
            new PersistentServerSetup
            ( 
                suite,
                "org.jacorb.test.imrutility.imr.Server",
                null,
                args
            );
 
        suite.addTest (new UDPTest("testUDP", setup));
        
        // Return the setup, not the suite!
        return setup;
    }
 
    protected void tearDown()
    {
        System.err.println("********** tearDown for testUDP");
        try
        {
            if (proc != null)
            {
                proc.destroy();
            }
         
            if (rep != null)
            {       
                rep.shutdown(false);
            }
        
            TestUtility.removeIMRFiles();
            TestUtility.removeFile(serverIOR);
            TestUtility.destroyImR();
            Thread.sleep(1000);
        }
        catch (Exception ex)
        {
            /* don't do anything */
        }                            
    }
    
    public void testUDP()
    {
        String [] args = null;
        String [] actArgs = null;
        StringBuffer command = null;
        BufferedReader br = null;
        TestServer test = null;
        ImplInfo [] info= null;
        CommandInfo cinfo = null;

        /* Set the file locations for the ImR not running
         * at the default location
         */
        String imrdatafile = home + separator + "altdata.dat";
        String imriorfile = home + separator + "altimr.ior";
        String imrbakfile = home + separator + "altbak.dat";
 
        ServerName sname = new ServerName();
        sname.impl("test_imr");
 
        /* Set the environment properties */         
        Environment.setProperty("jacorb.imr.ior_file", imriorfile);
        Environment.setProperty("jacorb.imr.imr_data_file", imrdatafile);
        Environment.setProperty("jacorb.imr.imr_backup_file", imrbakfile);
        Environment.setProperty("ORBInitRef.ImplementationRepository", "file:///" + imriorfile);                                              
        Environment.setProperty("jacorb.imr.udp_port", "23456");
        
        //Start the IMR        
        try
        {                     
            args = new String[11];
            args[0] = "-p";
            args[1] = "23451";            
            args[2] = "-f";
            args[3] = imrdatafile;
            args[4] = "-i";
            args[5] = imriorfile;
            args[6] = "-b";
            args[7] = imrbakfile;
            args[8] = "-u";
            args[9] = "23456";
            args[10] = "ORBInitRef.ImplementationRepository=file:///" + imriorfile;
                        
            rep = TestUtility.startImR(args, null);
            
            assertTrue("Failed to get ImR", rep != null);            
        }
        catch(Exception ex)
        {
            fail("Exception getting ImR" + ex.getMessage());
        }

        /* get command to start server, start the server - gets IOR file*/        
        try
        {
            command = setup.getStartCommand();
            serverIOR = home + separator + ior;
            command.append(" -i ");
            command.append(serverIOR);
                                    
            /* Need to wait longer because the ImRLocator waits 3 seconds"*/
            proc = TestUtility.startServer(command, 10000);     
            
            assertTrue("Server not registered", rep.listServers(sname).length == 1);
            
        }
        catch (IOException ioe)
        {
            fail("Got IOException starting server " + ioe.getMessage());
        }
        catch (Exception ex)
        {
            fail("Got Exception starting server " + ex.getMessage());
        }        
        
        try
        {        
            br = new BufferedReader(new FileReader(serverIOR));

            // get object reference from command-line argument file
            org.omg.CORBA.Object obj =
                TestUtility.getImRORB().string_to_object(br.readLine());

            br.close();

            //narrow to correct interface
            test = TestServerHelper.narrow(obj);
                        
            test.ping();                        
        }
        catch (Exception ex)
        {
            fail("Unexpected exception - server should be running");
        }
    }
}
