package org.jacorb.test.orb.dynany;

import junit.framework.*;
import junit.extensions.TestSetup;
import org.omg.CORBA.Any;
import org.omg.CORBA.TCKind;
import org.omg.CORBA.TypeCode;
import org.omg.DynamicAny.DynAny;
import org.omg.DynamicAny.DynSequence;

import org.jacorb.test.common.ORBSetup;
import org.jacorb.test.Bound;
import org.jacorb.test.ArrayTypeHelper;

/**
 * DynAnyObjectTest.java
 *
 * DynAny tests for embededded object type.
 *
 */

public class DynAnyObjectTest extends TestCase
{
   private static org.omg.DynamicAny.DynAnyFactory dynFactory = null;
   private static org.omg.CORBA.ORB orb = null;

   private static String nameService =
      "IOR:000000000000002B49444C3A6F6D672E6F72672F436F734E616D696E672F4E616"
      + "D696E67436F6E746578744578743A312E30000000000002000000000000007400010"
      + "2000000000E3231332E34382E39312E31353700836C0000001F5374616E646172644"
      + "E532F4E616D655365727665722D504F412F5F726F6F7400000000020000000000000"
      + "008000000004A414300000000010000001C000000000001000100000001050100010"
      + "00101090000000105010001000000010000002C00000000000000010000000100000"
      + "01C00000000000100010000000105010001000101090000000105010001";

   public DynAnyObjectTest (String name)
   {
      super (name);
   }

   public static Test suite ()
   {
      TestSuite suite = new TestSuite ("DynAny Object Test");
      Setup setup = new Setup (suite);
      ORBSetup osetup = new ORBSetup (setup);

      suite.addTest (new DynAnyObjectTest ("testInsertDynAnyObject"));
      return osetup;
   }

    public void testInsertDynAnyObject()
    {
        Any a = null;
        DynAny dyn_any = null;

        org.omg.CORBA.Object  obj = orb.string_to_object (nameService);

        TypeCode tc = org.omg.CosNaming.NamingContextExtHelper.type();

        try
        {
            dyn_any =
                 (DynAny)
                     dynFactory.create_dyn_any_from_type_code (tc);
        }
        catch (Exception ex)
        {
            fail ("Failed to create DynAny : " + ex);
        }

        try
        {
           dyn_any.insert_reference (obj);
        }
        catch (Exception ex)
        {
            fail ("Failed to insert object reference : " + ex);
        }

        try
        {
            a = dyn_any.to_any();
        }
        catch (Exception ex)
        {
            fail ("Exception dyn_sequence.to_any on sequence of Objects : " + ex);
        }

        dyn_any.destroy();
    }


    private static class Setup extends TestSetup
    {
        public Setup (Test test)
        {
            super (test);
        }

        protected void setUp ()
        {
            org.omg.CORBA.Object obj = null;

            orb = ORBSetup.getORB ();

            try
            {
                obj = orb.resolve_initial_references ("DynAnyFactory");
            }
            catch (org.omg.CORBA.ORBPackage.InvalidName ex)
            {
                fail ("Failed to resolve DynAnyFactory: " + ex);
            }

            try
            {
                dynFactory = org.omg.DynamicAny.DynAnyFactoryHelper.narrow (obj);
            }
            catch (Throwable ex)
            {
                fail ("Failed to narrow to DynAnyFactory: " + ex);
            }
        }

        protected void tearDown ()
        {
        }
    }
}
