package org.jacorb.test.poa;

import org.omg.CORBA.*;
import org.omg.PortableServer.*;
import org.omg.PortableServer.POAPackage.*;
import junit.framework.*;
import org.jacorb.test.orb.BasicServerImpl;
import org.jacorb.test.BasicServer;
import org.jacorb.test.BasicServerHelper;


public class Deactivate extends TestCase
{
   public Deactivate (String name)
   {
      super (name);
   }

   public static Test suite()
   {
      TestSuite suite = new TestSuite ("Deactivation");

      suite.addTest (new Deactivate ("test_deactivate"));
      suite.addTest (new Deactivate ("test_deactivate_activator"));

      return suite;
   }

   public static void test_deactivate ()
   {
      ORB orb = null;
      POA root, system = null;
      byte[] id1, id2;

      try
      {
         orb = ORB.init(new String[0], null);
         root = POAHelper.narrow(orb.resolve_initial_references( "RootPOA" ));

         // create POA
         Policy policies[] = new Policy[3];
         policies[0] = root.create_id_assignment_policy(
            org.omg.PortableServer.IdAssignmentPolicyValue.SYSTEM_ID);
         policies[1] = root.create_id_uniqueness_policy(
            org.omg.PortableServer.IdUniquenessPolicyValue.UNIQUE_ID);
         policies[2] = root.create_servant_retention_policy(
            org.omg.PortableServer.ServantRetentionPolicyValue.RETAIN);

         system = root.create_POA("system_id", root.the_POAManager(), policies);
      }
      catch(AdapterAlreadyExists ex)
      {
         fail( "unexpected exception: " + ex );
      }
      catch(InvalidPolicy ex)
      {
         fail( "unexpected exception: " + ex );
      }
      catch(Exception  ex)
      {
         fail( "unexpected exception: " + ex );
      }

      // create Servants
      Test_impl servant1 = new Test_impl();
      Test_impl servant2 = new Test_impl();
      // first activate servants
      try
      {
         id1 = system.activate_object(servant1);
         id2 = system.activate_object(servant2);

         // deactivate the servants now
         // no request is pending
         try
         {
            system.deactivate_object(id2);
            system.deactivate_object(id1);

            // now again try to deactivate
            // I would expect ObjectNotActive Exception but didn't get one
            try
            {
               system.deactivate_object(id1);
               fail( "deactivate_object called twice, expecting ObjectNotActive exception, but didn't");
            }
            catch(ObjectNotActive ex)
            {
               System.out.println("Success - caught ObjectNotActive");
            }
            catch(WrongPolicy ex)
            {
               fail( "unexpected exception: " + ex );
            }
         }
         catch(ObjectNotActive ex)
         {
            fail( "unexpected exception: " + ex );
         }
         catch(WrongPolicy ex)
         {
            fail( "unexpected exception: " + ex );
         }
      }
      catch(ServantAlreadyActive ex)
      {
         fail( "unexpected exception: " + ex );
      }
      catch(WrongPolicy ex)
      {
         fail( "unexpected exception: " + ex );
      }
   }


    /**
     * <code>test_deactivate_activator</code> is a test for JAC2 - When
     * ServantActivators with a RETAIN policy are used local object
     * invocations do not activate the object so a deactivate fails.
     */
   public static void test_deactivate_activator ()
   {
      ORB orb = null;
      POA root, system = null;
      byte[] id1, id2;

      try
      {
         orb = ORB.init(new String[0], null);
         root = POAHelper.narrow(orb.resolve_initial_references( "RootPOA" ));
         root.the_POAManager().activate();

         // create POA
         Policy policies[] = new Policy[2];
         policies[0] = root.create_servant_retention_policy(
            org.omg.PortableServer.ServantRetentionPolicyValue.RETAIN);
         policies[1] = root.create_request_processing_policy(
             RequestProcessingPolicyValue.USE_SERVANT_MANAGER);

         system = root.create_POA("system_id", root.the_POAManager(), policies);
         system.the_POAManager().activate();
         system.set_servant_manager(new PoaServantActivator());

         org.omg.CORBA.Object objectRef = system.create_reference(BasicServerHelper.id());
         BasicServer ref = BasicServerHelper.narrow(objectRef);
         // Local op; will incarnate the object.
         ref.ping();

         // Now try deactivating it.
         system.deactivate_object (system.reference_to_id(ref));
      }
      catch(Exception  ex)
      {
         fail( "unexpected exception: " + ex );
      }
   }



    /**
     * <code>PoaServantActivator</code> is a ServantActivator for
     * test_deactivate_activator.
     */
   static class PoaServantActivator extends LocalObject implements ServantActivator
   {
       // The incarnate operation is invoked by the POA whenever the POA receives
       // a request for an object that is not currently active, assuming the POA
       // has the RETAIN and USE_SERVANT_MANAGER policies
       public Servant incarnate(byte[] oid, POA adapter) throws ForwardRequest
       {
           try
           {
               BasicServerImpl servantObj = new BasicServerImpl();
               org.jacorb.util.Debug.output
               (
                   "PoaServantActivator.incarnate(): Created \"" +
                   servantObj.getClass().getName() + "\" " +
                   "servant object for \"" + adapter.the_name() +
                   "\""
               );
               return servantObj;
           }
           catch (Exception e)
           {
               System.err.println("incarnate: Caught exception - " + e);
           }
           return null;
       }

       public void etherealize(byte[] oid, POA adapter, Servant serv,
                               boolean cleanup_in_progress,
                               boolean remaining_activations)
       {
           org.jacorb.util.Debug.output("PoaServantActivator.etherealize()");
       }
   }
}
