/* JNDI service tables */

CREATE TABLE OF_JNDI
(
   SID NUMBER NOT NULL,          /* server id uuid string */
   OID CHAR(36) NOT NULL,        /* object id uuid string */
   Name VARCHAR2(255) NOT NULL,  /* name string */
   Class VARCHAR2(255) NOT NULL, /* classname */
   ChildUUID CHAR(36) NOT NULL,  /* leaf or context child uuid */
   PRIMARY KEY (SID, OID, Name)
)
;

CREATE TABLE OF_JNDI_Objects
(
   ChildUUID CHAR(36) NOT NULL,  /* object id uuid string */
   Object LONG RAW NOT NULL,     /* serialized Object */
   PRIMARY KEY (ChildUUID)
)
;

/* Not using this currently */
/* CREATE INDEX OF_JNDI_Index ON OF_JNDI(SID, OID, Name, ChildUUID); */


CREATE PROCEDURE of_jndi_delete_context
(
   ids IN NUMBER,
   ido IN VARCHAR2,
   idc IN VARCHAR2,
   tag OUT NUMBER
)
IS
BEGIN
   tag := 0;
   UPDATE OF_JNDI SET CHILDUUID = '00000000-0000-0000-0000-000000000000' WHERE CHILDUUID = (SELECT CHILDUUID FROM OF_JNDI WHERE SID = ids AND OID = ido AND NAME = idc);
   DELETE FROM OF_JNDI WHERE SID = ids AND OID = ido AND NAME = idc;
   IF SQL%FOUND THEN
      tag := 1;
   END IF;
END of_jndi_delete_context;
/


CREATE PROCEDURE of_jndi_delete_objects
(
   ids IN NUMBER,
   ido IN VARCHAR2,
   idc IN VARCHAR2,
   tag OUT NUMBER
)
IS
BEGIN
   tag := 0;
   DELETE FROM OF_JNDI_OBJECTS WHERE CHILDUUID = (SELECT CHILDUUID FROM
           OF_JNDI WHERE SID = ids AND OID = ido AND NAME = idc);
   DELETE FROM OF_JNDI WHERE SID = ids AND OID = ido AND NAME = idc;
   IF SQL%FOUND THEN
      tag := 1;
   END IF;
END of_jndi_delete_objects;
/


CREATE PROCEDURE of_jndi_insert_context
(
   ids IN NUMBER,
   ido IN VARCHAR2,
   idc IN VARCHAR2,
   classname IN VARCHAR2,
   child IN VARCHAR2
)
IS
BEGIN
   INSERT INTO OF_JNDI VALUES (ids, ido, idc, classname, child);
END of_jndi_insert_context;
/

CREATE PROCEDURE of_jndi_insert_object
(
   ids IN NUMBER,
   ido IN VARCHAR2,
   idc IN VARCHAR2,
   classname IN VARCHAR2,
   child IN VARCHAR2,
   obj IN LONG RAW
)
IS
BEGIN
   INSERT INTO OF_JNDI VALUES (ids, ido, idc, classname, child);
   INSERT INTO OF_JNDI_OBJECTS VALUES (child, obj);
END of_jndi_insert_object;
/


CREATE PROCEDURE of_jndi_update_context
(
   ids IN NUMBER,
   ido IN VARCHAR2,
   idc IN VARCHAR2,
   typ IN VARCHAR2,
   child IN VARCHAR2,
   tag OUT NUMBER
)
IS
BEGIN
   tag := 0;
   UPDATE OF_JNDI SET CHILDUUID = child, CLASS = typ
      WHERE SID = ids AND OID = ido AND NAME = idc;
   IF SQL%FOUND THEN
      tag := 1;
   END IF;
END of_jndi_update_context;
/

CREATE PROCEDURE of_jndi_update_object
(
   ids IN NUMBER,
   ido IN VARCHAR2,
   idc IN VARCHAR2,
   typ IN VARCHAR2,
   obj IN LONG RAW,
   tag OUT NUMBER
)
IS
   ch VARCHAR2(36);
BEGIN
   tag := 0;
   UPDATE OF_JNDI SET CLASS = typ
      WHERE SID = ids AND OID = ido AND NAME = idc;
   IF SQL%FOUND THEN
      tag := 1;
      /* If we found something to update we can continue safely */
      SELECT CHILDUUID INTO ch FROM OF_JNDI
         WHERE SID = ids AND OID = ido AND NAME = idc;
      UPDATE OF_JNDI_OBJECTS SET OBJECT = obj WHERE CHILDUUID = ch;
      IF SQL%NOTFOUND THEN
         INSERT INTO OF_JNDI_Objects VALUES (ch, obj);
      END IF;
   END IF;
END of_jndi_update_object;
/

CREATE PROCEDURE of_jndi_get_object
(
   ids IN NUMBER,
   ido IN VARCHAR2,
   idc IN VARCHAR2,
   outclass OUT VARCHAR2,
   outchild OUT VARCHAR,
   outobj OUT LONG RAW
)
IS
BEGIN
   /* Set the obj to null in case we're not using it */
   outobj := null;
   SELECT CLASS, CHILDUUID INTO outclass, outchild FROM OF_JNDI WHERE
      SID = ids AND OID = ido AND NAME = idc;
   IF outclass != 'com.prismt.j2ee.jndi.OpenFusionSPI' THEN
      SELECT OBJECT INTO outobj FROM OF_JNDI_OBJECTS WHERE
      CHILDUUID = outchild;
   END IF;
EXCEPTION
   WHEN NO_DATA_FOUND
   THEN
      outclass := null;
      outchild := null;
      outobj   := null;
END of_jndi_get_object;
/
