package org.jacorb.imrutility.imr.persistence;

import java.io.Serializable;
import java.util.HashMap;

import org.jacorb.imrutility.imr.CommandData;
import org.jacorb.imrutility.imr.POANameKey;
import org.jacorb.util.Debug;



/**
 * @author cj
 * This class holds the start up command details registered with the
 * {@link Activator}
 */
public class ActivatorData implements Serializable
{
    /**
     * Table of commands registered with the Activator
     */
    HashMap commands;
   
    /**
     * Constructor
     */
    public ActivatorData()
    {
        commands = new HashMap();        
    }
    
    /**
     * Adds a command.  
     * @param <code>pnkey</code> a key to the impl/poa name(s)
     * @param <code>cmd</code> the command details for starting the server
     */
    public void addCommand(POANameKey pnkey, CommandData cmd)
    {
        commands.put(pnkey, cmd);
    }

    /**
     * Removes a command for a given impl/POA
     * @param <code>pnkey</code> a key to the impl/poa name(s)
     */
    public void removeCommand(POANameKey pnkey)
    {
        commands.remove(pnkey);
    }
    
    /**
     * Returns the command registered with this Activator
     */
    public HashMap getCommands()
    {
        return commands;
    }
    
    /**
     * Implemented from the Serializable interface. For 
     * automatic initializing after deserialization.
     */
    private void readObject(java.io.ObjectInputStream in)
        throws java.io.IOException, java.io.NotActiveException, 
            ClassNotFoundException
    {
        in.defaultReadObject();
    }    
    
}
