package org.jacorb.imrutility.imr.policy;

import org.jacorb.imrutility.imr.LoadBalancingPolicy;

import java.util.HashMap;
import java.util.TreeMap;

/**
 * @author cj
 *
 * This class applies the Last In First Out load balancing policy.
 * It returns the active server that was the latest registered
 */
public class LIFO implements LBPolicy
{

    /**
     * Returns the unique id for the LIFO policy
     */
    public int policy()
    {
        return LoadBalancingPolicy._LIFO;
    }

    /**
     * This method returns the last registered server that is active
     * if no active servers are found then the first registered 
     * inactive server is returned.  A <code>TreeMap</code> is used as
     * this keeps the data in the order entered e.g. the first registered
     * server will be the first entry, the last registered server will be
     * the last entry.
     * @param name the implementation name - not used by this class
     * @param servers a Map containing the time the server was registered
     * as the key and the Server ID
     */
    public HashMap getSelectedServer (String name, TreeMap servers)
    {      
        HashMap selected = new HashMap();
        selected.put(servers.lastKey(), servers.get(servers.lastKey()));
        return selected;
    }
}


