package org.jacorb.imrutility.imr.policy;


import org.jacorb.imrutility.imr.LoadBalancingPolicy;

import java.util.Arrays;
import java.util.Collections;
import java.util.HashMap;
import java.util.TreeMap;
import java.util.List;
import java.util.Map;


/**
 * @author cj
 *.
 * This class applies the Random load balancing policy.
 * It selects an active server at random.
 */
public class Random implements LBPolicy
{

    /**
     * Returns the unique id for the Random policy
     */
    public int policy()
    {
        return LoadBalancingPolicy._RANDOM;
    }

    /**
     * This method returns a server selected at random. 
     * As only servers from the active table are included there is no
     * check to determine if the server is really active
     * @param name the implementation name - not used by this class
     * @param servers a Map containing the time the server was registered
     * as the key and the id of the Server
     */   
    public HashMap getSelectedServer (String name, TreeMap servers)
    {
        Map.Entry entry = null;
        HashMap selected = new HashMap();
        
        List choices;
//        Collection sinfos = servers.values();
    
        if (servers.size() > 0)
        {
            choices = Arrays.asList (servers.entrySet().toArray());
            Collections.shuffle (choices);
        
            entry = (Map.Entry) choices.get(0);
        }
              
        selected.put(entry.getKey(), entry.getValue());
        
        return selected;
    }
}
