package org.jacorb.imrutility.imr.policy;

import org.jacorb.imrutility.imr.LoadBalancingPolicy;
import org.jacorb.imrutility.imr.ServerInfo;
import org.jacorb.imrutility.imr.ServerState;

import java.util.HashMap;
import java.util.Collection;
import java.util.TreeMap;
import java.util.Map;

/**
 * @author cj
 *
 * This class applies the RoundRobin load balancing policy.
 * It iterates through the registered servers, in order, to 
 * obtain the next server.
 */
public class RoundRobin implements LBPolicy
{
   /**
    * Map holding position at which to start search for
    * each recorded implementation.  The implementation name is
    * the key.
    */
   private HashMap lastPositionMap = new HashMap();
   
   /**
    * Returns the unique id for the RoundRobin policy
    */
   public int policy()
   {
      return LoadBalancingPolicy._ROUND_ROBIN;
   }

   /**
    * This method selects a server in order of registration.  No check is 
    * made as to whether the Server is active as the Repository is 
    * configured to check the Aliveness according to a given policy.  The
    * check is made by the Repository
    * @param name the implementation name - not used by this class
    * @param servers a Map containing the time the server was registered
    * as the key and the ID of the Server
    */      
   public HashMap getSelectedServer (String name, TreeMap servers)
   {
      Integer lastPos;
      int pos;
      int tries = 0;
      Object [] serverArray;
      HashMap selected = new HashMap();
      Map.Entry entry;
                  
      if (servers.size() > 0)
      {
         serverArray = servers.entrySet().toArray();

         lastPos = (Integer) lastPositionMap.get (name);

         if (lastPos == null)
         {
            // first use of this group
            pos = 0;
         }
         else
         {
            pos = lastPos.intValue ();
            
            /* The number of servers may have reduced since the
             * last time if a server has become inactive.  If the
             * position is greater than the last server position then 
             * reset to the first server
             */
            if (pos > servers.size() - 1)
            {
                pos = 0;
            }
         }
                       
         entry = (Map.Entry) serverArray [pos];
            
         Thread.currentThread().interrupted ();
             
         pos = (pos + 1) % serverArray.length;
            
         lastPositionMap.put (name, new Integer (pos));
         
         selected.put(entry.getKey(), entry.getValue());
      }
      
      return selected;
      
   }
}