package org.jacorb.imrutility.imr.util;

import java.net.DatagramSocket;
import java.net.DatagramPacket;
import java.net.InetAddress;
import java.net.SocketException;

import java.util.ArrayList;

import org.apache.avalon.framework.logger.*;

import org.jacorb.util.Debug;

/**
 * @author cj
 *
 * This class handles sending of DatagramPackets using the UserDatagramProtocol
 * (UDP).  With this protocol there is no connection and it is unreliable.  The
 * class is a utility class to be used to send requests to specified ports.
 * Where sending of the request is successful the subsequent processing will be
 * handled by a different process.
 *
 */
public class MulticastClient
{
    int port = 0;
    DatagramSocket s = null;
    ArrayList returnPackets = new ArrayList();
    Logger logger = Debug.getNamedLogger("jacorb.multicast");     
   
    /**
     * Constructor
     * @param <code>port</code> the port on which the MulticastServer is
     * running
     */
    public MulticastClient (int port)
        throws Exception
    {
        this.port = port;
        s = new DatagramSocket ();
    }

    /**
     * Method that sends the request to the specified port
     * @param <code>data</code> the data to be used to identify the request
     */
    public void multicastSend (byte [] data)
        throws Exception
    {
        DatagramPacket returned;
        long waitTime = 0;
            
        InetAddress group = InetAddress.getByName ("230.0.0.1");

        DatagramPacket dgp = new DatagramPacket (data, data.length, group ,port);

        try
        {
            s.setSoTimeout(3000);
        }
        catch (SocketException se)
        {
            logger.warn("ERROR :: got exception setting socket timeout", se);
        }
      
        s.send(dgp);  

        waitTime = System.currentTimeMillis() + 3000;
        
        /* Configurable ? arbitrary at present*/  
        while (System.currentTimeMillis() < waitTime)
        {
            try
            {
                returned = new DatagramPacket(new byte[1024], 1024);

                s.receive(returned); 

                returnPackets.add(returned);
            }
            catch (Exception ex)
            {
                if (logger.isDebugEnabled())
                {
                    logger.debug("MulticastClient timedout");  
                }
            }
        }                                
    }
   
    /**
     * Method to get the data returned back from the request
     */
    public byte [] [] getReturnedData()
    {
        byte [] [] returnedData = new byte [returnPackets.size()] [];    
             
        for (int i = 0; i < returnPackets.size(); i++)
        {
            returnedData[i] =  ( (DatagramPacket) returnPackets.get(i)).getData();
        }
        
        return returnedData;
    }

   /**
    * Method to close this socket when action is complete
    */
   public void close()
   {
      s.close();
   }
}
