package org.jacorb.orb;

/*
 *        JacORB - a free Java ORB
 *
 *   Copyright (C) 1999-2003 Gerald Brose
 *
 *   This library is free software; you can redistribute it and/or
 *   modify it under the terms of the GNU Library General Public
 *   License as published by the Free Software Foundation; either
 *   version 2 of the License, or (at your option) any later version.
 *
 *   This library is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *   Library General Public License for more details.
 *
 *   You should have received a copy of the GNU Library General Public
 *   License along with this library; if not, write to the Free
 *   Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 */

import org.omg.ETF.Profile;
import org.omg.IOP.TaggedComponent; 

/**
* Extends the extensible transport framework profile definition to
* permit the marshalling of the AddressProfile data only.
* <p>
* Profiles that implement this interface will be capable of being 
* registered with the IMR.
* <p>
* There are a number of additions in this interface above and beyond what
* is required for a Profile by the version of ETF that JacORB's code is based on.
* This is because we will need to build an object ref outside of the marshal
* method that is supported by the (old) ETF spec when creating an IMRified
* object ref for a server and within the IMR when creating a location forward
* IOR.
* <p>
* In a nutshell, an IOP::TaggedProfile.profile_data can be populated
* by combining the results of the three methods on this interface
* with an object key octet sequence as per the following algorithm:
* <li> Create a CDR encapsulation as per the encapsulation type from {@link #encapsulation()}.
* <li> byte-by-byte copy the result of {@link #getAddressProfile()} into the encapsulation.
* <li> Marshal the object key sequence into the encapsulation.
* <li> Marshal the union of {@link #getTaggedComponents()} and any other per ORB / POA
* Tagged Components into the encapsulation.
* <li> The buffer of your encapsulation now contains the profile_data. 
* @author Simon McQueen
* @version $Id: ImRReadyProfile.java,v 1.3 2004/03/09 13:56:18 sm Exp $
*/

public interface ImRReadyProfile extends Profile
{
   public static final short BIG_ENDIAN_TAG_INTERNET_IOP = 0;
   
   public static final short LITTLE_ENDIAN_TAG_INTERNET_IOP = 1;
   
   public static final short ALREADY_COMPLETE = 2; 
   
   /**
   * Indicates the encapsulation that will be used by this profile
   * when encoding it's AddressProfile bytes, and which should subsequently
   * be used when marshalling all the rest of the TaggedProfile.profile_data.
   */
   public short encapsulation();
   
   /**
   * Return the bytes that would make up the ETF::AddressProfile bytes
   * (will include the endianess indicator).
   */
   public byte[] getAddressProfile();
   
   /**
   * Return any IOP::TaggedComponents that are personal to this Profile / endpoint.
   */
   public TaggedComponentList getComponents();
}
