package org.jacorb.orb.giop;

import org.omg.IOP.*;
import org.omg.ETF.*;

import org.jacorb.orb.CDROutputStream;
import org.jacorb.orb.ImRReadyProfile;
import org.jacorb.orb.TaggedComponentList;

/**
 * @author Simon McQueen
 * @version $Id: MultipleComponentProfile.java,v 1.2 2004/05/05 09:46:23 rnc Exp $
 */
public class MultipleComponentProfile extends ProfileBase
                         implements Cloneable
{
    public MultipleComponentProfile()
    {
        version = new org.omg.GIOP.Version ((byte)1, (byte)2);
    }

    public MultipleComponentProfile(byte[] data)
    {
        version = new org.omg.GIOP.Version ((byte)1, (byte)2);
        components = new TaggedComponentList(data);
    }

    public MultipleComponentProfile(org.omg.IOP.TaggedComponent[] multipleComponents)
    {
        version = new org.omg.GIOP.Version ((byte)1, (byte)2);
        components = new TaggedComponentList();
        for (int i = 0; i < multipleComponents.length; i++)
        {
            components.addComponent(multipleComponents[i]);
        }
    }

    public MultipleComponentProfile(TaggedComponentList components)
    {
        version = new org.omg.GIOP.Version ((byte)1, (byte)2);
        this.components = components;
    }

    public void marshal (TaggedProfileHolder tagged_profile,
                         TaggedComponentSeqHolder components)
    {
        if (encapsulation() != 0)
        {
            // You're going to have to define your own marshal operation
            // for littleEndian profiles.
            // The CDROutputStream only does big endian.
            // @todo Suitable error.
        }

        // Start a CDR encapsulation for the profile_data
        CDROutputStream profileDataStream = new CDROutputStream();
        profileDataStream.beginEncapsulatedArray();

        // Write the length of the TaggedProfile sequence.
        profileDataStream.write_long(this.components.size() + components.value.length);

        // Write the TaggedProfiles (ours first, then the ORB's)
        for (int i = 0; i < this.components.asArray().length; i++)
        {
            TaggedComponentHelper.write(profileDataStream, this.components.asArray()[i]);
        }
        for (int i = 0; i < components.value.length; i++)
        {
            TaggedComponentHelper.write(profileDataStream, components.value[i]);
        }

        // Populate the TaggedProfile for return.
        tagged_profile.value = new TaggedProfile
        (
            this.tag(),
            profileDataStream.getBufferCopy()
        );

        profileDataStream.close();
    }

    public Profile copy()
    {
        try
        {
            return (Profile)this.clone();
        }
        catch (CloneNotSupportedException e)
        {
            throw new RuntimeException ("error cloning profile: " + e);
        }
    }

    /**
     * This function shall determine if the passed profile, prof, is a match
     * to this profile.  The specifics of the match are left to the details
     * of the underlying transport, however profiles shall be considered a
     * match, if they would create connections that share the same attributes
     * relevant to the transport setup.  Among others, this could include
     * address information (eg. host address) and transport layer
     * characteristics (eg. encryption levels). If a match is found, it
     * shall return true, or false otherwise.
     */
    public boolean is_match(Profile prof)
    {
        return false;
    }

    public int tag()
    {
        return TAG_MULTIPLE_COMPONENTS.value;
    }

    /**
    * Return the bytes that would make up the ETF::AddressProfile bytes (new spec).
    * N.B. In this case *does* include the endianess indicator because the ecapsulation style is 3.
    */
    public byte[] getAddressProfile()
    {
        CDROutputStream addressProfileStream = new CDROutputStream();
        addressProfileStream.beginEncapsulatedArray();
        // Write the length of the TaggedProfile sequence.
        addressProfileStream.write_long(this.components.size());

        // Write the Multiple Components
        for (int i = 0; i < this.components.asArray().length; i++)
        {
            TaggedComponentHelper.write(addressProfileStream, this.components.asArray()[i]);
        }
        return addressProfileStream.getBufferCopy();
    }

    public short encapsulation()
    {
        return ImRReadyProfile.ALREADY_COMPLETE;
    }
}