package org.jacorb.test.imrutility.imr;

import java.io.File;
import org.jacorb.imrutility.imr.*;
import org.jacorb.util.Environment;
import java.util.Properties;

/**
 * This is  a utility class used to perform common functions
 * required by all the ImR tests
 */
public class TestUtility
{
    private static Process imrProcess;
    private static Process actProcess;
    protected static org.omg.CORBA.ORB imrOrb;
    protected static org.omg.CORBA.ORB actOrb;                  
                    
    public static RepositoryAdmin startImR
    (
        String [] args,
        Properties clientOrbProperties
    )
    throws Exception
    {
        if (clientOrbProperties == null)
        {            
            clientOrbProperties = new Properties ();
        }

        clientOrbProperties.setProperty
            ("org.omg.CORBA.ORBClass", "org.jacorb.orb.ORB");
        clientOrbProperties.setProperty
            ("org.omg.CORBA.ORBSingletonClass", "org.jacorb.orb.ORBSingleton");

        imrOrb = org.omg.CORBA.ORB.init (args, clientOrbProperties);
                        
        int count = 0;
        
        StringBuffer sb = new StringBuffer();
        
        if (args.length > 0)
        {
            for (int i = 0; i < args.length; i++)
            {
                sb.append(' ');
                sb.append(args[i]);               
            }
        }

        String command = "jaco org.jacorb.imrutility.imr.RepositoryImpl" + sb.toString();
                
        imrProcess = Runtime.getRuntime().exec(command);
                
        File f = new File(Environment.getProperty("jacorb.imr.ior_file"));        
            
        /* Wait up to 5 seconds for the IOR file to be written*/
        while (!f.exists() && count < 6)
        {
            System.err.println("File doesn't exist and count is " + count);
            count++;
            Thread.sleep(5000);
        }
                        
        RepositoryAdmin rep = RepositoryAdminHelper.narrow
            (imrOrb.resolve_initial_references("ImplementationRepository"));              
                 
        return rep;
    }  
 
    public static org.omg.CORBA.ORB getImRORB()
    {
        return imrOrb;
    }    

    public static org.omg.CORBA.ORB getActORB()
    {
        return actOrb;
    }    
   
    public static Process startServer(StringBuffer command)
        throws Exception
    {
        return startServer(command, 3000);               
    }
    
    public static Process startServer(StringBuffer command, int wait)
        throws Exception
    {       
        Process proc = Runtime.getRuntime().exec(command.toString());  
        
        /* Wait for the IOR file to be written & Server registered*/
        Thread.sleep(wait);                         
        
        return proc;
    }
    
    public static void removeIMRFiles()
        throws Exception
    {
        /* Make sure old files have been deleted before we start*/
        removeFiles(getImRFiles());        
    }
    
    private static String [] getImRFiles()
    {
        return new String [] 
            {Environment.getProperty("jacorb.imr.ior_file"),
             Environment.getProperty("jacorb.imr.imr_data_file"),
             Environment.getProperty("jacorb.imr.imr_backup_file")};             
    }
 
    private static String [] getActFiles()
    {
        return new String [] 
            {Environment.getProperty("jacorb.imr.activator_ior_file"),
             Environment.getProperty("jacorb.imr.act_data_file"),
             Environment.getProperty("jacorb.imr.act_backup_file")};             
    }
    
    public static void removeFile(String file)
        throws Exception
    {
        String [] files = new String[1];    
        
        files[0] = file;
        removeFiles(files);
    }
        
    public static void removeFiles(String [] files)
        throws Exception
    {
        File f = null;
        
        for (int i = 0; i < files.length; i++)
        {
            f = new File(files[i]);
            
            if (f.exists())
            {
                f.delete();
            }     
        }    
        
        Thread.sleep(3000);
    }
    
    public static void destroyImR()
    {
        if (imrProcess != null)
        {
           imrProcess.destroy();
        }
    }   
    
    public static Activator startActivator    
    (
        String [] args,
        Properties props
    )
    throws Exception
    {
        int count = 0;

        if (props == null)
        {            
            props = new Properties ();
        }

        props.setProperty
            ("org.omg.CORBA.ORBClass", "org.jacorb.orb.ORB");
        props.setProperty
            ("org.omg.CORBA.ORBSingletonClass", "org.jacorb.orb.ORBSingleton");

        actOrb = org.omg.CORBA.ORB.init (args, props);
        
        StringBuffer sb = new StringBuffer();
        
        if (args.length > 0)
        {
            for (int i = 0; i < args.length; i++)
            {
                sb.append(' ');
                sb.append(args[i]);               
            }
        }
        
        String command = "jaco org.jacorb.imrutility.imr.ActivatorImpl" + sb.toString();
        
        actProcess = Runtime.getRuntime().exec(command);
                
        File f = new File(Environment.getProperty("jacorb.imr.activator_ior_file"));
        
        /* Wait up to 5 seconds for the IOR file to be written*/
        while (!f.exists() && count < 6)
        {
            System.err.println("Activator File doesn't exist and count is " + count);
            count++;
            Thread.sleep(1000);
        }
                        
        Activator act = ActivatorHelper.narrow
            (actOrb.resolve_initial_references("IMRActivator"));              
                 
        return act;        
    }
    
    public static void stopActivator()
    {
        if (actProcess != null)
        {
            actProcess.destroy();
        }
    } 
    
    public static void removeActivatorFiles()
        throws Exception
    {
        /* Make sure old files have been deleted before we start*/        
        removeFiles(getActFiles());        
    }                   
}
